function [p,tet,iters,detailinf] = CB_distmesh3d_mpc(fFunc,fH,h_max,h_min,bbox,pfix,pmpc,endtol,MPCType,updateControl,PercentageMPC,varargin)
%DISTMESH2D 2-D Mesh Generator using Distance Functions.
%   [P,T] = CB_distmesh2d_mpc(fFunc,fH,h_max,h_min,bbox,pfix,pmpc,endtol,varargin
%
%      P:         Node positions (Nx2)
%      T:         Triangle indices (NTx3)
%      fFunc:     Distance function d(x,y)
%      FH:        Scaled edge length function h(x,y)
%      h_max:     MaxEdge length(associated fH function)
%      h_min:     MaxEdge length(associated fH function)
%      BBOX:      Bounding box [xmin,ymin; xmax,ymax]
%      PFIX:      Fixed node positions (NFIXx2)
%      PMPC:      Any initiated MPC points
%      Endtol:    Termination tolerence for Newton step

%   See also: MESHDEMO2D, DISTMESHND, DELAUNAYN, TRIMESH.

%   distmesh2d.m v1.1
%   Copyright (C) 2004-2012 Per-Olof Persson. See COPYRIGHT.TXT for details.
tic

%% Set up parameters
ttol = 0.2; % Change tolerence for p movement
geps = 0.;  % This the boudary limit for eliminating points and triangles
densityctrlfreq = 1; % destiny control frequency
destL = 1.2; % Length factor for length control on elements
UpdateLimiter = updateControl; % p update limiter This enforces stability for highly tensioned meshes
AllowNewts = 3; % Newtons teps allowed before remeshing
ploton  = true; %Plot during evaluations (see mesh updates

%% MPC Method selection STAYS THE SAME

if MPCType ==  1 %La grange
    MPC_lagrange = 1; % 1 use la grange MPC 0 use ellimination
    EllimACt = 0; % Actual 1 approx 0 for Elimination method only
elseif MPCType ==  2 %Ellim actual
    MPC_lagrange = 0; % 1 use la grange MPC 0 use ellimination
    EllimACt = 1; % Actual 1 approx 0 for Elimination method only
else %Ellim approx
    MPC_lagrange = 0; % 1 use la grange MPC 0 use ellimination
    EllimACt = 0; % Actual 1 approx 0 for Elimination method only
end

%% 1. Create initial distribution in bounding box (equilateral triangles) COMPLETED CONVERSION to 3d

h02 = 0.8*(0.1*h_max + .9*h_min); %initial mesh seed edge length
h02 = 0.9*h_max;


[x,y,z] = meshgrid(bbox(1,1):h02:bbox(2,1),bbox(1,2):h02*sqrt(3)/2:bbox(2,2),bbox(1,3):h02*sqrt(6)/3:bbox(2,3));
x(2:2:end,:,:) = x(2:2:end,:,:) + h02/2; % Shift even xz planes in x
y (:,:,2:2:end) = y(:,:,2:2:end) + h02*1/sqrt(3); % Shift even xy planes in x to create the tetrahedrals
p = [x(:),y(:),z(:)]; % List of all node coordinates

%% 2. Remove points outside the region, apply the rejection method STAYS THE SAME

p = p(fFunc.fd(p)<+geps.*fH(p),:);

% r0 = 1./feval(fH,p,varargin{:}).^2;                  % Probability to keep point
% p = p(rand(size(p,1),1)<r0./max(r0),:);              % Rejection method

if ~isempty(pfix), p = setdiff(p,pfix,'rows'); end     % Remove duplicated nodes
if ~isempty(pmpc), p = setdiff(p,pmpc,'rows'); end     % Remove duplicated nodes cb2

pfix = unique(pfix,'rows'); nfix = size(pfix,1);

%% Find unique mpc points from the input values (remove Fixed nodes)
pmpc = [pfix; pmpc];
[ ~, IA ,~] =   unique(pmpc,'rows','first');
IA = sort(IA,'ascend');
pmpc = pmpc(IA);
pmpc = pmpc(nfix+1:end,:); nmpc = size(pmpc,1);

%% Create enumeration types 0 free, 1 fixed, 2 mpc, 3 forcesnap
s = 4;
typeENUM.fixed = 0+s;
typeENUM.mpc = 1+s;
typeENUM.free = 2+s;
typeENUM.forcesnap = -1;

% Ensure unique fixed nodes
p = [pfix; pmpc; p];                                        % Prepend fix points
% p = unique(p,'rows','stable');

[~, IA, ~] = unique(p,'rows','first'); %keep unique points selecting first retains the Pfix, and PMC numbers
IA = sort(IA,'ascend');
p = p(IA,:);


FixMpcTypes = [typeENUM.fixed*ones(size(pfix,1),1); typeENUM.mpc*ones(size(pmpc,1),1)];
types = typeENUM.free*ones(size(p,1),1);
types(1:size(FixMpcTypes,1)) = FixMpcTypes;

% For first variable initialisation
counter = 0;
counter2 = 0;
pold = inf;
removedpcount = 0;
addedpcount = 0;
minqual = 0;
densitynewts = inf;
newtonsteps = inf;
ForcedRemesh = inf;
ForceNewts = false;
remesh = false;
allowsize = true
while 1
    counter = counter +1;
    ReMeshedAtIter(counter,1) = 0;
    
    if ForceNewts ==  false
        %% 3. Retriangulation by the Delaunay algorithm
        if ((max(sqrt(sum((p-pold).^2,2))./feval(fH,p)) > ttol) && (newtonsteps > AllowNewts) && (minqual < 0.1)) || (ForcedRemesh >0)          % Any large movement?
            tic
            
            [p, In] = sortrows(p,'ascend');
            types = types(In,:);
            
            [~,IA,~] = unique(p,'rows','first');
            IA = sort(IA,'ascend'); % Keep the rows stable
            p = p(IA,:);
            types = types(IA);
            
            p = real(p);
                                                      % Save current positions
            tet = delaunay(p);                        % List of triangles
            %% Find low quality triangle
            quals = simpqual(p,tet);
            lowquals = find(quals <= 0.1);
            
            if (size(lowquals,1) > 0) && (ForcedRemesh == 0)
                newps = zeros(0,2);
                numlowtris  = size(lowquals,1);
                newps = (p(tet(lowquals,1),:)+p(tet(lowquals,2),:)+p(tet(lowquals,3),:)+p(tet(lowquals,4),:))/4; % add mid points
                p = [p; newps];
                types = [types; ones(numlowtris,1)*typeENUM.free];
                tet = delaunay(p);
                
                pmid = (p(tet(:,1),:)+p(tet(:,2),:)+p(tet(:,3),:)+p(tet(:,4),:))/4;    % Compute triangle centroids
%                 tet = tet(fFunc.fd(pmid)<-(geps.*fH(pmid)),:);         % Keep interior triangles only
            end
            
            pold = p;
            
            %% 4. Describe each bar by a unique pair of nodes
            bars = [tet(:,[1,2]);tet(:,[1,3]);tet(:,[1,4]);tet(:,[2,3]);tet(:,[2,4]);tet(:,[3,4])]; % Interior bars duplicated
            bars = sort(bars,2);
            bars = unique(bars,'rows'); %sorts and orders nodes
            
            faces = [tet(:,[1,2,3]);
                tet(:,[1,2,4]);
                tet(:,[1,3,4]);
                tet(:,[2,3,4]);];                        
            
            faces = sort(faces,2);
            [faces, ~, ic] = unique(faces,'rows');
            
            %  Add the triangles on the surface of the mesh to the MPC list to force a
            %  step to the edge of the solver
  
            % 5. Graphical output of the current mesh
            if ploton ==  true
                MESHplotter3D(p,tet,MPCType)
%                 hold on;plot3(p(SurfPs,1),p(SurfPs,2),p(SurfPs,3),'g.');hold off
            end
            remesh = true;
            ForcedRemesh = 0;% Cb2
            RemeshTimer = toc;
        end
        
   
    end
    %% 6. Create bar lengths
    if (remesh ==  true || (mod(newtonsteps,AllowNewts) == 0))&& ForceNewts == false   % Recalculate bar lengths every allowed newton step, this allows for quadratic convergence
        barvec = p(bars(:,1),:) - p(bars(:,2),:);            % List of bar vectors
        L = sqrt(sum(barvec.^2,2));                          % L = current bar lengths
        hbars = feval(fH,(p(bars(:,1),:)+p(bars(:,2),:))/2,varargin{:}); % Actual desired lengths
%         L0 = hbars*sqrt(sum(L.^2)/sum(hbars.^2)); L0act = L0;    % L0 = Desired lengths scaled by sum of the squares
%         L0 = hbars*sqrt(sum(L.^2)/sum(hbars.^2)); L0act = hbars;
        L0 = hbars; L0act = hbars;
        disp('LENGTH CHANGED')
    end

    %% CB2 Start solve code variable initialisation
    types(:,1)= typeENUM.free;
    types(1:4,1) = typeENUM.fixed;
    nRan = round(PercentageMPC/100*(size(types,1)),0);
    if nRan ~= 0
      types(end-nRan:end,:) = typeENUM.mpc;
    end
   
    if remesh ==  true
        % number of mpc nodes lagrange
        MPCnums = find(types == typeENUM.mpc);
        nmpc = size(MPCnums,1);
        MPCdof = sort([MPCnums*3-2; MPCnums*3-1; MPCnums*3]);
        
        % find number of fixed nodes
        pnums = find(types == typeENUM.fixed);
        pdof = sort([pnums*3-2;pnums*3-1;pnums*3]);
        
        % find number of free nodes
        fnums = find(types == typeENUM.free);
        fdof = sort([fnums*3-2;fnums*3-1;fnums*3]);
        ndof = size(MPCdof,1)+size(pdof,1)+size(fdof,1); %truss contribution total dofs
        
        u = zeros(ndof,1);
        
        lambda = 0;
        
        remesh = false;
        newtonsteps = 0;
    end
    
    %% Truss Consistant tangent contributions
    nTimes = 3;
    times = 0;
    for tims = 1:nTimes
        F = zeros(ndof,1);
        DFDU = sparse(ndof,ndof);
        tic;
        %     ForceNewts = true;
        for i = 1:size(bars,1)
            dof = [bars(i,1)*3-2, bars(i,1)*3-1,bars(i,1)*3-0, bars(i,2)*3-2, bars(i,2)*3-1, bars(i,2)*3-0]';
            L0bar = L0(i,1); %desired length of bar
            uloc = u(dof);
            x = [pold(bars(i,1),:), pold(bars(i,2),:)]';
            [dFdu,broken,Ft] = CB_3dtruss_CT(uloc,x,L0bar); % CB_2dtruss_CT has 3x methods for Bar contribution.
            
            if broken ==  true
                break
            end
            
            DFDU(dof,dof) = DFDU(dof,dof) + dFdu;
            
            F(dof) = F(dof) + Ft;
        end
        times = times + toc;
    end
     TrussContributionTimer(counter,1) = times/nTimes;
    
    %% MPC methods contributions
    % These mehtods can be selected at the input parameters at the top of this script
    % La grange and Ellimination methids have seperate newton step updates
    K_global = sparse(0,0);
    
    
    
    if 1 == MPC_lagrange %% Lagrange for circle
        times = 0;
        for tims = 1:nTimes
            mpccount = 0;
            DRmpcdUc = sparse(nmpc,ndof);
            DDRmpcddUc = zeros(ndof,ndof);
            RMpc = sparse(nmpc,1);
            
            tic;
            
            if newtonsteps ==  0 %just remeshed
                lambda = zeros(nmpc,1); %only used for la grange
                ldof = ((ndof+1):(ndof+nmpc))';
            end
            
            for i = MPCnums'
                mpccount = mpccount +1;
                
                dof = i*3-2:i*3;
                
                pcur = pold(i,:);
                uloc = u(dof);
                
                Point2B = pcur + uloc';
                
                DRmpcdUc(mpccount,dof) = fFunc.dF(Point2B);
                
                ddRddu = fFunc.ddF(Point2B);
                
                DDRmpcddUc(dof,dof) = DDRmpcddUc(dof,dof)+lambda(mpccount,1)*ddRddu;
                
                RMpc(mpccount,1) = fFunc.F(Point2B);
            end
            
            K_global = [(DFDU+DDRmpcddUc) DRmpcdUc'
                DRmpcdUc sparse(nmpc,nmpc)];
            
            RHS = sparse(ndof+nmpc,1);
            RHS(pdof,1) = F(pdof,1);
            RHS(fdof,1) = F(fdof,1);
            RHS(ldof,1) = RMpc;
            RHS(MPCdof,1) = F(MPCdof,1)+(DRmpcdUc(:,MPCdof))'*lambda;
            times = times + toc;
        end
        MpcContributionTimer(counter,1) = times/nTimes;
        %solve dof
        solvedof = [fdof;MPCdof;ldof];
        
        % Newton update
        Supdate = zeros(ndof+nmpc,1);
        tic
        nTimes = 10;
        for j = 1:nTimes
    %           Supdate(solvedof) = -K_global(solvedof,solvedof)\RHS(solvedof);
                   Supdate(solvedof) = -minres(K_global(solvedof,solvedof),RHS(solvedof),10^-3,100);

        end
%         Supdate(solvedof) = -minres(K_global(solvedof,solvedof),RHS(solvedof),10^-3,100);
        SystemSolveTimer(counter,1) = toc/nTimes;
        figure(70+MPCType)
%         spy(K_global)
        SystemNonzeros(counter,1) = nnz(K_global);
        spy(K_global(solvedof,solvedof));
        
        MPCResolveCounter(counter,1) = 0;
        
        % Reshape to same as p
        update = Supdate(1:ndof,1);
        delta_lam = Supdate(ldof);
        
        %     %Min Norm update attempt
        P_update = [update(1:3:end,1), update(2:3:end,1),update(3:3:end,1)];
        
        % Determine proportion of update to be used
        % if any bar wants to change by more than half its length, then reduce
        % update size
        
        barvecupdate = P_update(bars(:,1),:)-P_update (bars(:,2),:);
        barchange = sqrt(sum(barvecupdate.^2,2));
        change = abs(barchange./L);
        maxchange = max(change);
        
        if maxchange > UpdateLimiter
            delta = UpdateLimiter/maxchange;
        else
            delta = 1;
        end
        
        lambda = lambda + delta*delta_lam;
        u = u + delta*update;
        p = pold+[u(1:3:end),u(2:3:end),u(3:3:end)];
        
    else
        %% Elimination methods
        %Selection is done by the slave selection as the partial
        %derivitive with the greatest constribution.
        times = 0;
        for tims = 1:nTimes
            Slavedofs_temp = sparse(ndof,1);
            Masterdofs_temp = sparse(ndof,1);
            Pmat_temp = sparse(ndof,ndof);
            dPdm_mat_temp = sparse(ndof,ndof);
            mpccount = 0;
            tic;
            for i = MPCnums'
                mpccount = mpccount +1;
                
                dof = i*3-2:i*3;
                
                pcur = pold(i,:);
                uloc = u(dof);
                
                Point2B = pcur + uloc';
                
                dRdu = fFunc.dF(Point2B);
                
                dRdu1 = dRdu(1,1);
                dRdu2 = dRdu(1,2);
                dRdu3 = dRdu(1,3);
                
                if 0 ==  EllimACt % CB2 second partial dirivitive approximation
                    
                    ddRddu = fFunc.ddF(Point2B);
                    if (abs(dRdu1) >=  abs(dRdu2)) && (abs(dRdu1) >=  abs(dRdu3))
                        SlaveLclDof = 1;
                        MasterLclDof = [2 3]; %master is dof2 dof3
                        
                    elseif (abs(dRdu2) >=  abs(dRdu1)) && (abs(dRdu2) >=  abs(dRdu3))
                        SlaveLclDof = 2;
                        MasterLclDof = [1 3]; %master is dof1 dof3
                    elseif (abs(dRdu3) >=  abs(dRdu1)) && (abs(dRdu3) >=  abs(dRdu2))
                        SlaveLclDof = 3;
                        MasterLclDof = [1 2]; %master is dof2 dof3
                    end
                    
                    MasterDof = dof(1,MasterLclDof); %master is dof1 dof3
                    SlaveDof = dof(1,SlaveLclDof);
                    
                    dRds = dRdu(1,SlaveLclDof);
                    dRdm1 = dRdu(1,MasterLclDof(1,1));
                    dRdm2 = dRdu(1,MasterLclDof(1,2));
                    
                    Pmatlocal = [-dRdm1/dRds, -dRdm2/dRds];
                    
                    dsdm1 = Pmatlocal(1,1);
                    dsdm2 = Pmatlocal(1,2);
                    
                    ddRdsds = ddRddu(SlaveLclDof,SlaveLclDof);
                    ddRdsdm1 = ddRddu(SlaveLclDof,MasterLclDof(1,1));
                    ddRdsdm2 = ddRddu(SlaveLclDof,MasterLclDof(1,2));
                    ddRdm1ds = ddRddu(MasterLclDof(1,1),SlaveLclDof);
                    ddRdm1dm1 = ddRddu(MasterLclDof(1,1),MasterLclDof(1,1));
                    ddRdm1dm2 = ddRddu(MasterLclDof(1,1),MasterLclDof(1,2));
                    ddRdm2ds = ddRddu(MasterLclDof(1,2),SlaveLclDof);
                    ddRdm2dm1 = ddRddu(MasterLclDof(1,2),MasterLclDof(1,1));
                    ddRdm2dm2 = ddRddu(MasterLclDof(1,2),MasterLclDof(1,2));
                    
                    % dd1d2d2 = -(ddRdu2du2*dRdu1 - dRdu2*ddRdu1du2)/(dRdu1^2);
                    % dd1d2d3 = -(ddRdu2du3*dRdu1 - dRdu2*ddRdu1du3)/(dRdu1^2);
                    % d1d3d2 = -(ddRdu3du2*dRdu1 - dRdu3*ddRdu1du2)/(dRdu1^2);
                    % dd1d3d3 = -(ddRdu3du3*dRdu1 - dRdu3*ddRdu1du3)/(dRdu1^2);
                    
                    ddsdm1dm1 = -((ddRdm1ds*dsdm1+ddRdm1dm1)*dRds - dRdm1*(ddRdsds*dsdm1+ddRdsdm1))/(dRds^2);
                    ddsdm1dm2 = -((ddRdm1ds*dsdm2+ddRdm1dm2)*dRds - dRdm1*(ddRdsds*dsdm2+ddRdsdm2))/(dRds^2);
                    ddsdm2dm1 = -((ddRdm2ds*dsdm1+ddRdm2dm1)*dRds - dRdm2*(ddRdsds*dsdm1+ddRdsdm1))/(dRds^2);
                    ddsdm2dm2 = -((ddRdm2ds*dsdm2+ddRdm2dm2)*dRds - dRdm2*(ddRdsds*dsdm2+ddRdsdm2))/(dRds^2);
                    
                    % ddsdm1dm1 = -((ddRdm1ds*dsdm1+ddRdm1dm1 )*dRds - dRdm1*(ddRdsds*dsdm1 + ddRdsdm1))/(dRds^2);
                    % ddsdm1dm2 = -((ddRdm1ds*dsdm2+ddRdm1dm2)*dRds - dRdm1*(ddRdsds*dsdm2+ddRdsdm2))/(dRds^2);
                    % ddsdm2dm1 = -((ddRdm2ds*dsdm2+ddRdm2dm1)*dRds - dRdm2*(ddRdsds*dsdm1+ddRdsdm1))/(dRds^2);
                    % ddsdm2dm2 = -((ddRdm2ds*dsdm2+ddRdm2dm2)*dRds - dRdm2*(ddRdsds*dsdm2+ddRdsdm2))/(dRds^2);
                    
                    dPmatlocal = [ddsdm1dm1 ddsdm1dm2;
                        ddsdm2dm1 ddsdm2dm2];
                    
                    Pmatlocal22 = fFunc.dFelimActual(Point2B,SlaveLclDof);
                    dPmatlocal22 = fFunc.ddFelimActual(Point2B,SlaveLclDof);
                    if norm(Pmatlocal - Pmatlocal22)> 1e-9 %|| norm(dPmatlocal-dPmatlocal22)> 1e-9
                        %fprintf('fail')
                        Failcount = 1;
                        
                    end
                    
                    %                 fprintf('test');
                else %actual second derivitive
                    if (abs(dRdu1) >=  abs(dRdu2)) && (abs(dRdu1) >=  abs(dRdu3))
                        sdof = 1;
                    elseif (abs(dRdu2) >=  abs(dRdu1)) && (abs(dRdu2) >=  abs(dRdu3))
                        sdof = 2;
                    elseif (abs(dRdu3) >=  abs(dRdu1)) && (abs(dRdu3) >=  abs(dRdu2))
                        sdof = 3;
                    end
                    SlaveDof = dof(1,sdof);
                    MasterDof= dof(1, setdiff(1:3,sdof)); %master is dof2
                    Pmatlocal = fFunc.dFelimActual(Point2B,sdof);
                    dPmatlocal = fFunc.ddFelimActual(Point2B,sdof);
                end
                
                
                Slavedofs_temp(SlaveDof,1) = 1;
                Masterdofs_temp(MasterDof,1) = 1;
                Pmat_temp(SlaveDof,MasterDof) = Pmatlocal;
                dPdm_mat_temp(MasterDof,MasterDof) = dPmatlocal*F(SlaveDof,1);
            end
            
            SlaveDofs = find(Slavedofs_temp == 1);
            MasterDofs = find(Masterdofs_temp == 1);
            
            %Solve degrees of freedom
            solvedof = [fdof;
                MasterDofs];
            
            Pmat = Pmat_temp(SlaveDofs,MasterDofs);
            dPdm_mat_term2 = dPdm_mat_temp(MasterDofs,MasterDofs);
            
            RHS_f = F(fdof,1);
            RHS_ms = F(MasterDofs,1) + Pmat'*F(SlaveDofs,1);
            RHS = [RHS_f;
                RHS_ms];
            
            Kff = DFDU(fdof,fdof);
            %         Kfm = DFDU(fdof,MasterDofs) + DFDU(fdof,SlaveDofs)*Pmat;
            Kmf = DFDU(MasterDofs,fdof) + Pmat'*DFDU(SlaveDofs,fdof);
            Kmm = DFDU(MasterDofs,MasterDofs) + dPdm_mat_term2 +  Pmat'*DFDU(SlaveDofs,MasterDofs) + DFDU(MasterDofs,SlaveDofs)*Pmat + Pmat'*DFDU(SlaveDofs,SlaveDofs)*Pmat;
            
            %Elmination Method mpcs K_global
            K_global = [Kff Kmf';
                Kmf Kmm];
            times = times + toc;
        end
        MpcContributionTimer(counter,1) = toc;
        % Newton update
        tic
        Supdate = zeros(ndof,1);
        for j = 1:nTimes
            L  = ichol((-K_global),struct('michol','on'));
%              Supdate(solvedof,1) = (-K_global)\RHS;%% Lagrange for circle
            Supdate(solvedof,1) = pcg(-K_global,RHS,10^-6,100,L,L');
        
        end
        SystemSolveTimer(counter,1) = toc/nTimes;
        SystemNonzeros(counter,1) = nnz(K_global);
        
        figure(70+MPCType);
        spy(K_global);
        maxchange = max(max(abs(Supdate(MasterDofs,1))))./0.1*h_max;
        delta =1;
        if maxchange > UpdateLimiter
            delta = UpdateLimiter/maxchange;
        else
            delta = 1;
        end
        Supdate(solvedof,1) = delta.*Supdate(solvedof,1);
        
        tic;
        
        for i = MPCnums'
            mpccount = mpccount +1;
            
            dof = i*3-2:i*3;
            pcurold = pold(i,:);
            ulocold = u(dof);
            
            PointPrev = pcurold + ulocold';
            
            if sum(dof(1) == MasterDofs) == 0
                lclslavedof = 1;
                lclmasterdof = [2 3];
            elseif sum(dof(2) == MasterDofs) == 0
                lclslavedof = 2;
                lclmasterdof = [1 3];
            else
                lclslavedof = 3;
                lclmasterdof = [1 2];
            end
                
            SupdateMaster = Supdate(dof(1,lclmasterdof),1);            
            SlaveUpdate = fFunc.FuncFelimActual(PointPrev,SupdateMaster,lclslavedof);            
            Supdate(dof(1,lclslavedof),1) = real(SlaveUpdate);
            
                        
            hold on
            mastup = zeros (1,3);
            mastup(1,lclmasterdof) = Supdate(dof(lclmasterdof),1)';
            
            ppplot = [PointPrev; PointPrev + mastup; PointPrev + Supdate(dof,1)'];
            
%             plot3(ppplot(:,1),ppplot(:,2),ppplot(:,3),'c-.');

%             k = [1 2];
%             plot3(ppplot( k,1),ppplot( k,2),ppplot( k,3),'c-');
%             
%             k = [2 3];
%             plot3(ppplot( k,1),ppplot( k,2),ppplot( k,3),'m-');
%             
%             k = [1 3];
%             plot3(ppplot(k,1),ppplot(k,2),ppplot(k,3),'r-');
            
        end
        MPCResolveCounter(counter,1) = toc;

% 
%          for i = MPCnums'
% %             mpccount = mpccount +1;
%             
%             dof = i*3-2:i*3;
%             pcurold = pold(i,:);
%             ulocold = u(dof);
%             
%             PointPrev = pcurold + ulocold';
%             
%             if sum(dof(1) == MasterDofs) == 0
%                 lclslavedof = 1;
%                 lclmasterdof = [2 3];
%             elseif sum(dof(2) == MasterDofs) == 0
%                 lclslavedof = 2;
%                 lclmasterdof = [1 3];
%             else
%                 lclslavedof = 3;
%                 lclmasterdof = [1 2];
%             end
%             
%             
%             %Update the slave usign the second order taylor
%             Pmatlocal = fFunc.dFelimActual(PointPrev,lclslavedof);
%             dPmatlocal = fFunc.ddFelimActual(PointPrev,lclslavedof);
%             
%             SupdateMaster = Supdate(dof(lclmasterdof),1);
%             
%             SlaveUpdate = Pmatlocal*SupdateMaster+ 0.5*SupdateMaster'*dPmatlocal*SupdateMaster;
%             Supdate(dof(1,lclslavedof),1) = real(SlaveUpdate);
%             
%             % Snap the points to the surface
%         end




        
        
        % Reshape to same as p
        update = Supdate(1:ndof,1);%% Lagrange for circle
        P_update = [update(1:3:end,1), update(2:3:end,1), update(3:3:end,1)];
        
        % Determine proportion of update to be used
        % if any bar wants to change by more than half its length, then reduce
        % update size
        
        barvecupdate = P_update(bars(:,1),:)-P_update (bars(:,2),:);
        barchange = sqrt(sum(barvecupdate.^2,2));
        change = abs(barchange./L);
        maxchange = max(change);
        
        if maxchange > UpdateLimiter
            delta = UpdateLimiter/maxchange;
        else
            delta = 1;
        end
        
        u = u + delta*update;
        p = pold+[u(1:3:end),u(2:3:end),u(3:3:end)];
        
    end
    
    newtonsteps = newtonsteps +1;
    
    %% end CB2
    
     % Plot lines that are used for the snap
     MESHplotter3D(pold,tet,MPCType)
     for ii = MPCnums'
         hold on
         lclp = [pold(ii,:); p(ii,:)];
         plot3(lclp(:,1),lclp(:,2),lclp(:,3),'r-')
     end
     
     title({'Master-Slave Elimination first update','No controls'})
     legend({'Mesh','MPC update vectors'})
    
    % 8. Termination criterion:
    
    MESHplotter3D(p,tet,MPCType)
    title({'Lagrangian first update','No controls','Mesh after update'})

    if ploton ==  true
        %PlotMPCs points around edge
        hold on
        
        plot3(p(MPCnums,1),p(MPCnums,2),p(MPCnums,3),'r.')
        plot3(p(pnums,1),p(pnums,2),p(pnums,3),'k*')
        
        drawnow
    end
    %     axis([-3 3 -3 3])
    axis equal
    hold off
    
    %    if max(abs(update)) < endtol
    qual = simpqual(p,tet,1);
    minqual = min(qual);
    NOrmupdt = norm(update)
    
        if NOrmupdt < 1
            AllowNewts = 10;
            densityctrlfreq = 5;
        end
        if NOrmupdt < .001
            ForceNewts = true
        end
    
%         if minqual > 0.6
%             ForceNewts = true;
%             types(find(types == typeENUM.mpc)) = typeENUM.fixed;
%             %AllowNewts = 3;
%             densityctrlfreq = 3;
%         else
%             %AllowNewts = 3;
%             densityctrlfreq = 5;
%         end
    NormUpdates(counter,1) = NOrmupdt;
    MpcCounter(counter,1) = nmpc;
    PCounter(counter,1) = size(p,1);
    SolveDofCounter(counter,1) = size(solvedof,1);    


    counter2 = counter2 +1;
    break
    if (NOrmupdt < endtol) || counter2 >= 50
        break;
    end
    
    %     if max(sqrt(sum(deltat*Ftot(d<-geps,:).^2,2))/h0)<dptol, break; end
end

detailinf.NormUpdates = NormUpdates;
detailinf.RemeshedAtIter = ReMeshedAtIter;
detailinf.RemovedPsCount = removedpcount;
detailinf.RemeshedTimes = RemeshTimer;
detailinf.TrussContributionTimer = TrussContributionTimer;
detailinf.MpcCounter = MpcCounter;
detailinf.PCounter = PCounter;
detailinf.SolveDofCounter = SolveDofCounter;

detailinf.MPCResolveCounter = MPCResolveCounter;
detailinf.SystemSolveTimes = SystemSolveTimer;
detailinf.MPCTimes = MpcContributionTimer;

detailinf.NonZeros = SystemNonzeros;

MESHplotter3D(p,tet,MPCType)

minqual = min(qual)

npsend = size(p,1)
removedpcount
addedpcount
iters  = counter

lr = L./L0act;

minls = min(lr)
maxls = max(lr)
avelr = mean(lr)
time = toc

% pause

% Clean up and plot final mesh
% % % [p,t] = fixmesh(p,t);
% % % simpplot(p,t)
end

function [a] = MESHplotter3D(p,tet,fignum)
figure(fignum); cla
% simpplot(p,tet,'p(:,2)>0');
% title(['Retriangulation #'])
%         drawnow
t = unique(sort([tet(:,[1,2,3]);tet(:,[1,2,4]);tet(:,[1,3,4]);tet(:,[2,3,4]);],2),'rows');
cla;
s1 = patch('vertices',p,'faces',t,'edgecol','k','facecol',[.8,.9,1]);
s1.FaceAlpha = 1.00;
axis equal

% draw low quality elements
qual = simpqual(p,tet,1);
AD = find(qual<.2);

t = unique(sort([tet(AD,[1,2,3]);tet(AD,[1,2,4]);tet(AD,[1,3,4]);tet(AD,[2,3,4]);],2),'rows');

% hold on
% s2 = patch('vertices',p,'faces',t,'edgecol','k','facecol','r'); drawnow;
% s2.FaceAlpha = 1.0;
% hold off

% r = 1.01;
% xo = 0;
% yo = 0;
% zo = 0;
% m =60;
% n = 60;
% theta=0:2*pi/m:2*pi ;
% phi=-pi/2:pi/n:pi/2 ;
% [T,P] = meshgrid(theta,phi) ;
% X = xo + r *cos(P).* cos(T);
% Y = yo + r *cos(P).* sin(T);
% Z = zo + r *sin(P) ;
% hold on
% s3 = surf(X,Y,Z); drawnow;
% set(s3, 'FaceAlpha', 0.5)
% set(s3, 'EdgeAlpha', 0.0)
% % shading interp
end


