function [p,tet,iters] = CB_distmesh3d_mpc(fFunc,fH,h_max,h_min,bbox,pfix,pmpc,endtol,MPCType,varargin)
%DISTMESH2D 2-D Mesh Generator using Distance Functions.
%   [P,T] = CB_distmesh2d_mpc(fFunc,fH,h_max,h_min,bbox,pfix,pmpc,endtol,varargin
%
%      P:         Node positions (Nx2)
%      T:         Triangle indices (NTx3)
%      fFunc:     Distance function d(x,y)
%      FH:        Scaled edge length function h(x,y)
%      h_max:     MaxEdge length(associated fH function)
%      h_min:     MaxEdge length(associated fH function)
%      BBOX:      Bounding box [xmin,ymin; xmax,ymax]
%      PFIX:      Fixed node positions (NFIXx2)
%      PMPC:      Any initiated MPC points
%      Endtol:    Termination tolerence for Newton step

%   See also: MESHDEMO2D, DISTMESHND, DELAUNAYN, TRIMESH.

%   distmesh2d.m v1.1
%   Copyright (C) 2004-2012 Per-Olof Persson. See COPYRIGHT.TXT for details.
tic

%% Set up parameters
ttol = 0.2; % Change tolerence for p movement
geps = 0.1;  % This the boudary limit for eliminating points and triangles
densityctrlfreq = 3; % destiny control frequency
destL = 1.5; % Length factor for length control on elements
UpdateLimiter = 20; % p update limiter This enforces stability for highly tensioned meshes
AllowNewts = 5; % Newtons teps allowed before remeshing
ploton  = true; %Plot during evaluations (see mesh updates

%% MPC Method selection STAYS THE SAME

if MPCType ==  1 %La grange
    MPC_lagrange = 1; % 1 use la grange MPC 0 use ellimination
    EllimACt = 0; % Actual 1 approx 0 for Elimination method only
elseif MPCType ==  2 %Ellim actual
    MPC_lagrange = 0; % 1 use la grange MPC 0 use ellimination
    EllimACt = 1; % Actual 1 approx 0 for Elimination method only
else %Ellim approx
    MPC_lagrange = 0; % 1 use la grange MPC 0 use ellimination
    EllimACt = 0; % Actual 1 approx 0 for Elimination method only
end

%% 1. Create initial distribution in bounding box (equilateral triangles) COMPLETED CONVERSION to 3d

% h02 = 0.8*(0.1*h_max + .9*h_min); %initial mesh seed edge length
h02 = 1*h_min;


[x,y,z] = meshgrid(bbox(1,1):h02:bbox(2,1),bbox(1,2):h02*sqrt(3)/2:bbox(2,2),bbox(1,3):h02*sqrt(6)/3:bbox(2,3));
x(2:2:end,:,:) = x(2:2:end,:,:) + h02/2; % Shift even xz planes in x
y (:,:,2:2:end) = y(:,:,2:2:end) + h02*1/sqrt(3); % Shift even xy planes in x to create the tetrahedrals
p = [x(:),y(:),z(:)]; % List of all node coordinates

%% 2. Remove points outside the region, apply the rejection method STAYS THE SAME

p = p(fFunc.fd(p)<+geps.*fH(p),:);

% r0 = 1./feval(fH,p,varargin{:}).^2;                  % Probability to keep point
% p = p(rand(size(p,1),1)<r0./max(r0),:);              % Rejection method

if ~isempty(pfix), p = setdiff(p,pfix,'rows'); end     % Remove duplicated nodes
if ~isempty(pmpc), p = setdiff(p,pmpc,'rows'); end     % Remove duplicated nodes cb2

pfix = unique(pfix,'rows'); nfix = size(pfix,1);

%% Find unique mpc points from the input values (remove Fixed nodes)
pmpc = [pfix; pmpc];
[ ~, IA ,~] =   unique(pmpc,'rows','first');
IA = sort(IA,'ascend');
pmpc = pmpc(IA);
pmpc = pmpc(nfix+1:end,:); nmpc = size(pmpc,1);

%% Create enumeration types 0 free, 1 fixed, 2 mpc, 3 forcesnap
s = 4;
typeENUM.fixed = 0+s;
typeENUM.mpc = 1+s;
typeENUM.free = 2+s;
typeENUM.forcesnap = -1;

% Ensure unique fixed nodes
p = [pfix; pmpc; p];                                        % Prepend fix points
% p = unique(p,'rows','stable');

[~, IA, ~] = unique(p,'rows','first'); %keep unique points selecting first retains the Pfix, and PMC numbers
IA = sort(IA,'ascend');
p = p(IA,:);


FixMpcTypes = [typeENUM.fixed*ones(size(pfix,1),1); typeENUM.mpc*ones(size(pmpc,1),1)];
types = typeENUM.free*ones(size(p,1),1);
types(1:size(FixMpcTypes,1)) = FixMpcTypes;

% For first variable initialisation
counter = 0;
counter2 = 0;
pold = inf;
removedpcount = 0;
addedpcount = 0;
minqual = 0;
densitynewts = inf;
newtonsteps = inf;
ForcedRemesh = inf;
ForceNewts = false;
remesh = false;
while 1
    counter = counter +1;
    if counter == 10
        disp('100')
    end
    if ForceNewts ==  false
%         ForceNewts = true;
%         load('test.mat')
        %% 3. Retriangulation by the Delaunay algorithm
        if ((max(sqrt(sum((p-pold).^2,2))./feval(fH,p)) > ttol) && (newtonsteps > AllowNewts) && (minqual < 0.1)) || (ForcedRemesh >0)          % Any large movement?
            
            [p, In] = sortrows(p,'ascend');
            types = types(In,:);
            
            [~,IA,~] = unique(p,'rows','first');
            IA = sort(IA,'ascend'); % Keep the rows stable
            p = p(IA,:);
            types = types(IA);
            
            p = real(p);
            % Save current positions
            tet = delaunay(p);                        % List of triangles
            
            pmid = (p(tet(:,1),:)+p(tet(:,2),:)+p(tet(:,3),:)+p(tet(:,4),:))/4;    % Compute triangle centroids
            tet = tet(fFunc.fd(pmid)<-(.01*fH(pmid)),:);         % Keep interior triangles only
            
            %% Find low quality triangle
            quals = simpqual(p,tet);
            lowquals = find(quals <= 0.1);
            
            % %             if (size(lowquals,1) > 0) && (ForcedRemesh == 0)
            % %                 newps = zeros(0,2);
            % %                 numlowtris  = size(lowquals,1);
            % %                 newps = (p(tet(lowquals,1),:)+p(tet(lowquals,2),:)+p(tet(lowquals,3),:)+p(tet(lowquals,4),:))/4; % add mid points
            % %                 p = [p; newps];
            % %                 types = [types; ones(numlowtris,1)*typeENUM.free];
            % %                 tet = delaunay(p);
            % %
            % %                 pmid = (p(tet(:,1),:)+p(tet(:,2),:)+p(tet(:,3),:)+p(tet(:,4),:))/4;    % Compute triangle centroids
            % %                 tet = tet(fFunc.fd(pmid)<-(geps.*fH(pmid)),:);         % Keep interior triangles only
            % %             end
            
            pold = p;
            
            %% 4. Describe each bar by a unique pair of nodes
            bars = [tet(:,[1,2]);tet(:,[1,3]);tet(:,[1,4]);tet(:,[2,3]);tet(:,[2,4]);tet(:,[3,4])]; % Interior bars duplicated
            bars = sort(bars,2);
            bars = unique(bars,'rows'); %sorts and orders nodes
            
            faces = [tet(:,[1,2,3]);
                tet(:,[1,2,4]);
                tet(:,[1,3,4]);
                tet(:,[2,3,4]);];
            
            faces = sort(faces,2);
            [faces, ~, ic] = unique(faces,'rows');
            
            %  Add the triangles on the surface of the mesh to the MPC list to force a
            %  step to the edge of the solver
            SurfFace =  find(histc(ic,min(ic):max(ic)) ==  1); % find unique triangles (edge only faces)
            SurfPs = unique(reshape(faces(SurfFace,:),{},1)); % find surface ps
            
            % Find all points on the boudary of the mesh and set them to
            % forced snap (unique triangles are boundary)
            
            MPCsurfPs = setdiff(SurfPs,[find(types == typeENUM.mpc);find(types == typeENUM.fixed)]);
            types(MPCsurfPs) = typeENUM.forcesnap;
            
            %Find all edge Tets and collect there ps including interior ps for
            % Checking of MPC conditions
            edgeTetss = zeros(0,1);
            for EdgeP = 1:size(SurfPs,1)
                p2find = SurfPs(EdgeP);
                edgeTetss = [edgeTetss;
                    find(tet(:,1) == p2find);
                    find(tet(:,2) == p2find);
                    find(tet(:,3) == p2find);
                    find(tet(:,4) == p2find)];
            end
            
            edgeTetss = unique(edgeTetss);
            EdgeCheckPs = unique(reshape(tet(edgeTetss,:),{},1),'rows'); % Create a list of all the Ps that are in edge triangles this speeds up the search process
            EdgeCheckPs = setdiff(EdgeCheckPs,find(types == typeENUM.fixed));
            
            % 5. Graphical output of the current mesh
            if ploton ==  true
                MESHplotter3D(p,tet,MPCType)
                hold on;plot3(p(SurfPs,1),p(SurfPs,2),p(SurfPs,3),'g.');hold off
            end
            if ForcedRemesh > 0
                disp(['Forced ' num2str(ForcedRemesh)])
            else
                disp('auto remeshed')
            end
            
            remesh = true;
            ForcedRemesh = 0;% Cb2
        end
        
        %% Snap points to boundaries if required (creates MPCs reference)
        
        % Check only Ps in edge triangles
        fDs = fFunc.fd(p(EdgeCheckPs,:));
        snapPoints = EdgeCheckPs(find(fDs >-0.5*feval(fH,p(EdgeCheckPs,:))),1);
        snapPoints = unique([snapPoints; find(types == typeENUM.forcesnap)]);
        
        %Find mpc points that are no longer close to the boudary
        NotMpcs = setdiff(find(types == typeENUM.mpc),snapPoints);
        types(NotMpcs,1) = typeENUM.free;
        
        snapPoints = setdiff(snapPoints,find(types == typeENUM.fixed));
        %%   snapPoints = setdiff(snapPoints,find(types == type.mpc));  %Removes mpc points from snapping
        if size(snapPoints,1) > 0 %%%%%%%%&& counter < 2 breaks it
            if size(find(types(snapPoints) ==  typeENUM.mpc),1) < size(snapPoints,1)
                types(snapPoints) = typeENUM.mpc;
                %                 cb2
                if counter<=10
                    pSnap = p(snapPoints,:);
                    dvecsnap = fFunc.distance(pSnap); % Shortest distance vec
                    
                    p(snapPoints,:) =  pSnap+ dvecsnap;
                    plotvecstobound  = 1;
                    if plotvecstobound ==  1 % Plot lines that are used for the snap
                        for ii = 1:size(snapPoints,1)
                            hold on
                            lclp = [pSnap(ii,:); (pSnap(ii,:)+ dvecsnap(ii,:))];
                            plot3(lclp(:,1),lclp(:,2),lclp(:,3),'r-')
                        end
                    end
                    pold = inf;
                    newtonsteps = inf;
                    ForcedRemesh = 1;
                    continue
                end
            end
        end
        
        %% cb2 remove external points not part of a bar
        % Still to evaluate neccesity
        
        p2remove = setdiff(1:size(p,1),bars)';
        if size(p2remove,1) > 0
            p(p2remove,:) = [];
            types(p2remove,:) = [];
            removedpcount =  removedpcount + size(p2remove,1);
            pold = inf;
            newtonsteps = inf;
            ForcedRemesh = 2;
            continue
        end
        
    end
    %% 6. Create bar lengths
    
    if remesh ==  true || (mod(newtonsteps,AllowNewts) == 0)   % Recalculate bar lengths every allowed newton step, this allows for quadratic convergence
        barvec = p(bars(:,1),:) - p(bars(:,2),:);            % List of bar vectors
        L = sqrt(sum(barvec.^2,2));                          % L = current bar lengths
        hbars = feval(fH,(p(bars(:,1),:)+p(bars(:,2),:))/2,varargin{:}); % Actual desired lengths
        %         L0 = hbars*sqrt(sum(L.^2)/sum(hbars.^2)); L0act = L0;    % L0 = Desired lengths scaled by sum of the squares
%                 L0 = 1.05*hbars*(sum(L.^2)/sum(hbars.^2))^(1/3); L0act = hbars;
        L0 = hbars; L0act = hbars;
        disp('LENGTH CHANGED')
    end
    
    %% CB2   Density control - remove/average points that are too close together
    if ForceNewts ==  false
        %% short edge density control STAYS THE SAME
        if ((mod(counter,densityctrlfreq)==0)  && any(L0act>destL*L)) || counter == 0 || any(L0act>3*destL*L)
            index = find(L0act>destL*L);
            ShrtLs = L(index,:);
            [~, InDx] = sort(ShrtLs,'ascend');
            shrtbars = bars(index,:);
            shrtbars = shrtbars(InDx,:);
            
            removedps = zeros(0,1);
            barnum = 0;
            
            while barnum < size(shrtbars,1)
                barnum =  barnum + 1;
                p1 = shrtbars(barnum,1);
                p2 = shrtbars(barnum,2);
                % Check if bar is to fixed node
                removedp = 0;
                if (types(p1,1) == typeENUM.fixed) && (types(p2,1) == typeENUM.fixed) % fixed fixed
                    % do nothing
                    continue
                elseif (types(p1,1) == typeENUM.fixed) && (types(p2,1) ~= typeENUM.fixed) % fixed & not fixed
                    % remove not fixed node
                    removedp = p2;
                elseif (types(p1,1) ~= typeENUM.fixed) && (types(p2,1) == typeENUM.fixed) % not fixed & fixed
                    % remove not fixed node
                    removedp = p1;
                elseif (types(p1,1) == typeENUM.mpc) && (types(p2,1) == typeENUM.mpc) % mpc &mpc
                    %take average point and make p1 as mpc and p2 removed
                    p(p1,:) = (p(p1,:)+p(p2,:))/2;
                    types(p1) = typeENUM.mpc;
                    removedp = p2;
                elseif (types(p1,1) == typeENUM.mpc) && (types(p2,1) ~= typeENUM.mpc) % mpc & not mpc
                    % leave mpc remove other point
                    removedp = p2;
                elseif (types(p1,1) ~= typeENUM.mpc) && (types(p2,1) == typeENUM.mpc) % not mpc & mpc
                    % leave mpc remove other point
                    removedp = p1;
                elseif (types(p1,1) == typeENUM.free) && (types(p2,1) == typeENUM.free) % free & free
                    %take average point and make p1 as free, p2 is removed
                    p(p1,:) = (p(p1,:)+p(p2,:))/2;
                    types(p1) = typeENUM.free;
                    removedp = p2;
                end
                
                if (removedp ~= 0) %overload control if zero, must not be considered
                    removedps =[removedps;removedp];
                end
                
                removedpcount = removedpcount +1;
                %rows containing the removed point
                rowsCR = unique([find(shrtbars(barnum+1:end,1)==removedp);find(shrtbars(barnum+1:end,2)==removedp)])+ barnum;
                
                if size(rowsCR,1) > 0
                    shrtbars(rowsCR,:) = []; % remove rows containing the removed point
                end
            end
            if size(removedps,1)>0
                p(removedps,:) = [];
                types(removedps,:) = [];
                
                pold=inf;
                newtonsteps = inf;
                ForcedRemesh = 3;
                continue;
            end
            
        end
        
        %Long edge handleing simple Low effectiveness
        
        %     if ((mod(count,densityctrlfreq) == 0 ) || ((Forcedestiny ==  1)) ) && any(L>destL./2*L0act)
        %         LongBars = bars(L>destL./2*L0act,:);
        %
        %         newps = (p(LongBars(:,1),:)+p(LongBars(:,2),:))/2;
        %
        %         newtypes = type.free*ones(size(newps,1),1);
        %         p = [p; newps];
        %         types = [types; newtypes];
        %
        %         addedpcount = addedpcount+ size(LongBars,1);
        %
        %         pold = inf;
        %         newtonsteps = inf;
        %         ForcedRemesh = true;
        %         continue;
        %     end
        
        %% Long bars edge control (add points) STAYS THE SAME
        testdest = sum(L)/sum(L0act);
        testdest = mean(L./L0act);
        if testdest < 1.4 && (testdest > 0.7 || any(L0act*destL*2 < L))&&false
            if ((mod(counter,densityctrlfreq)==0 )  && any(L>destL*L0act)) || any(L>3*destL*L0act)%
                
                index = find(L>destL*L0act);
                
                LongLs = L(index,:);
                LongL0acts = L0act(index,:);
                LongBars = bars(index,:);
                
                [LongLs,InDx] = sort(LongLs,'descend');
                LongL0acts = LongL0acts(InDx,:);
                LongBars = LongBars(InDx,:);
                
                newps = zeros(0,1);
                newTypes = zeros(0,1);
                barnum = 0;
                
                while barnum < size(LongBars,1)
                    barnum = barnum + 1;
                    p1 = LongBars(barnum,1);
                    p2 = LongBars(barnum,2);
                    
                    NoPsToAdd = floor(LongLs(barnum,:)/(destL*LongL0acts(barnum,1)));
                    %                 NoPsToAdd = floor(LongLs(barnum,:)/(LongL0acts(barnum,1)));
                    AddedPs = p(p1,:) + 1/(NoPsToAdd+1)*[1:NoPsToAdd]'.*(p(p2,:)-p(p1,:));
                    
                    %remove long bars that have an edge attached to this
                    AddedTypes = ones(NoPsToAdd,1)*typeENUM.free;
                    rowsCR = unique([find(LongBars(barnum+1:end,1)==LongBars(barnum,1));
                        find(LongBars(barnum+1:end,2)==LongBars(barnum,1));
                        find(LongBars(barnum+1:end,1)==LongBars(barnum,2));
                        find(LongBars(barnum+1:end,2)==LongBars(barnum,2))]);%+ barnum;
                    
                    if size(rowsCR,1) > 0
                        LongBars(rowsCR,:) = []; % remove rows containing the removed point
                        LongL0acts(rowsCR,:) = [];
                        LongLs(rowsCR,:) = [];
                    end
                    if size(AddedPs,1) ~= size(AddedTypes,1)
                        disp('failed to add types')
                        pause
                    end
                    newps = [newps; AddedPs];
                    newTypes = [newTypes; AddedTypes];
                end
                
                p = [p;newps];
                types = [types;newTypes];
                
                addedpcount = addedpcount+ size(newps,1);
                
                pold=inf;
                newtonsteps = inf;
                ForcedRemesh = 4;
                continue;
            end
        elseif testdest >= 1.4 &&false
            if ((mod(counter,densityctrlfreq)==0 )  && any(L>destL*L0act)) || any(L>3*destL*L0act)%
                
                side1 = sqrt(sum((p(tet(:,2),:)-p(tet(:,1),:)).^2,2));
                side2 = sqrt(sum((p(tet(:,3),:)-p(tet(:,1),:)).^2,2));
                side3 = sqrt(sum((p(tet(:,3),:)-p(tet(:,2),:)).^2,2));
                side4 = sqrt(sum((p(tet(:,4),:)-p(tet(:,1),:)).^2,2));
                side5 = sqrt(sum((p(tet(:,4),:)-p(tet(:,2),:)).^2,2));
                side6 = sqrt(sum((p(tet(:,4),:)-p(tet(:,3),:)).^2,2));
                
                tcircs = side1+side2+side3+side4+side5+side6;
                pmid = (p(tet(:,1),:)+p(tet(:,2),:)+p(tet(:,3))+p(tet(:,4),:))/4;    % Compute triangle centroids
                Largets = find(tcircs./6 > destL.*fH(pmid));
                
                newps = (p(tet(Largets,1),:)+p(tet(Largets,2),:)+p(tet(Largets,3))+p(tet(Largets,4),:))/4;
                
                if size(newps,1)>0
                    p = [p; newps];
                    types = [types; ones(size(newps,1),1)*typeENUM.free];
                    
                    pold=inf;
                    newtonsteps = inf;
                    ForcedRemesh = 42;
                    continue;
                end
            end
        end
        
        Forcedensity = 0;
    end
    %% CB2 Start solve code variable initialisation
    if remesh ==  true
        % number of mpc nodes lagrange
        MPCnums = find(types == typeENUM.mpc);
        nmpc = size(MPCnums,1);
        MPCdof = sort([MPCnums*3-2; MPCnums*3-1; MPCnums*3]);
        
        % find number of fixed nodes
        pnums = find(types == typeENUM.fixed);
        pdof = sort([pnums*3-2;pnums*3-1;pnums*3]);
        
        % find number of free nodes
        fnums = find(types == typeENUM.free);
        fdof = sort([fnums*3-2;fnums*3-1;fnums*3]);
        ndof = size(MPCdof,1)+size(pdof,1)+size(fdof,1); %truss contribution total dofs
        
        u = zeros(ndof,1);
        
        lambda = 0;
        
        remesh = false;
        newtonsteps = 0;
    end
    
    %% Truss Consistant tangent contributions
    F = zeros(ndof,1);
    ivec = zeros(size(bars,1)*36,1);
    jvec = zeros(size(bars,1)*36,1);
    vvec = zeros(size(bars,1)*36,1);
    tic;
    %     ForceNewts = true;
    vecloc = 0;
    for i = 1:size(bars,1)
        dof = [bars(i,1)*3-2, bars(i,1)*3-1,bars(i,1)*3-0, bars(i,2)*3-2, bars(i,2)*3-1, bars(i,2)*3-0]';
        L0bar = L0(i,1); %desired length of bar
        uloc = u(dof);
        x = [pold(bars(i,1),:), pold(bars(i,2),:)]';
        [dFdu,broken,Ft] = CB_3dtruss_CT(uloc,x,L0bar); % CB_2dtruss_CT has 3x methods for Bar contribution.
        
        if broken ==  true
            break
        end
        
        for ibas = 1:size(dof,1);
            is = dof(ibas);
            for jbas = 1:size(dof,1);
                js = dof(jbas);
                vecloc = vecloc + 1;
                ivec(vecloc,1) = is;
                jvec(vecloc,1) = js;
                vvec(vecloc,1) = dFdu(ibas,jbas);
            end
        end
        
        %             DFDU(dof,dof) = DFDU(dof,dof) + dFdu;
        
        F(dof) = F(dof) + Ft;
    end
    DFDU = sparse(ivec,jvec,vvec);
    
    
    %% MPC methods contributions
    % These mehtods can be selected at the input parameters at the top of this script
    % La grange and Ellimination methids have seperate newton step updates
    
    if 1 == MPC_lagrange %% Lagrange for circle
        mpccount = 0;
        DRmpcdUc = sparse(nmpc,ndof);
        DDRmpcddUc = sparse(ndof,ndof);
        RMpc = sparse(nmpc,1);
        
        if newtonsteps ==  0 %just remeshed
            lambda = zeros(nmpc,1); %only used for la grange
            ldof = ((ndof+1):(ndof+nmpc))';
        end
        
        for i = MPCnums'
            mpccount = mpccount +1;
            
            dof = i*3-2:i*3;
            
            pcur = pold(i,:);
            uloc = u(dof);
            
            Point2B = pcur + uloc';
            
            DRmpcdUc(mpccount,dof) = fFunc.dF(Point2B);
            
            ddRddu = fFunc.ddF(Point2B);
            
            DDRmpcddUc(dof,dof) = DDRmpcddUc(dof,dof)+lambda(mpccount,1)*ddRddu;
            
            RMpc(mpccount,1) = fFunc.F(Point2B);
        end
        
        K_global = [(DFDU+DDRmpcddUc) DRmpcdUc'
            DRmpcdUc sparse(nmpc,nmpc)];
        
        RHS = sparse(ndof+nmpc,1);
        RHS(pdof,1) = F(pdof,1);
        RHS(fdof,1) = F(fdof,1);
        RHS(ldof,1) = RMpc;
        
        
        RHS(MPCdof,1) = F(MPCdof,1)+(DRmpcdUc(:,MPCdof))'*lambda;
        
        %solve dof
        solvedof = [fdof;MPCdof;ldof];
        nTimes = 1; 
        % Newton update
        tic
        Supdate = zeros(ndof+nmpc,1);
        eas = -K_global(solvedof,solvedof)\RHS(solvedof);
        toc
        
%         gamma = normest((DFDU+DDRmpcddUc))/normest(DRmpcdUc)^2;
%         W = speye(nmpc)*gamma;
%         K_global = [(DFDU+DDRmpcddUc+DRmpcdUc'*W*DRmpcdUc) DRmpcdUc'
%                 DRmpcdUc sparse(nmpc,nmpc)];
%        
%         RHS(1:ndof,1) = RHS(1:ndof,1)+DRmpcdUc'*W*RMpc;
% 
% 
%         Amat = K_global(solvedof,solvedof);
%         bvec = RHS(solvedof);
%         
%         tic
%         nTimes = 1;
%         for j = 1:nTimes
% %               Supdate(solvedof) = -K_global(solvedof,solvedof)\RHS(solvedof);
% %               Supdate(solvedof) = -Amat\bvec;
%               eas2 = -minres(Amat,bvec,10^-10,10000);
%         end
%         Supdate(solvedof) = -minres(K_global(solvedof,solvedof),RHS(solvedof),10^-3,100);
        SystemSolveTimer(counter,1) = toc/nTimes;
        
        Supdate = zeros(ndof+nmpc,1);
        Supdate(solvedof) = eas;
              
        
        % Reshape to same as p
        update = Supdate(1:ndof,1);
        delta_lam = Supdate(ldof);
        
        %     %Min Norm update attempt
        P_update = [update(1:3:end,1), update(2:3:end,1),update(3:3:end,1)];
        
        % Determine proportion of update to be used
        % if any bar wants to change by more than half its length, then reduce
        % update size
        
        barvecupdate = P_update(bars(:,1),:)-P_update (bars(:,2),:);
        barchange = sqrt(sum(barvecupdate.^2,2));
        change = abs(barchange./L);
        maxchange = max(change);
        
        if maxchange > UpdateLimiter
            delta = UpdateLimiter/maxchange;
        else
            delta = 1;
        end
        
        lambda = lambda + delta*delta_lam;
        u = u + delta*update;
        p = pold+[u(1:3:end),u(2:3:end),u(3:3:end)];
        
    else
        %% Elimination methods
        %Selection is done by the slave selection as the partial
        %derivitive with the greatest constribution.
        
        Slavedofs_temp = sparse(ndof,1);
        Masterdofs_temp = sparse(ndof,1);
        Pmat_temp = sparse(ndof,ndof);
        dPdm_mat_temp = sparse(ndof,ndof);
        mpccount = 0;
        for i = MPCnums'
            mpccount = mpccount +1;
            
            dof = i*3-2:i*3;
            
            pcur = pold(i,:);
            uloc = u(dof);
            
            Point2B = pcur + uloc';
            
            dRdu = fFunc.dF(Point2B);
            
            dRdu1 = dRdu(1,1);
            dRdu2 = dRdu(1,2);
            dRdu3 = dRdu(1,3);
            
            if 0 ==  EllimACt % CB2 second partial dirivitive approximation
                
                ddRddu = fFunc.ddF(Point2B);
                if (abs(dRdu1) >=  abs(dRdu2)) && (abs(dRdu1) >=  abs(dRdu3))
                    SlaveLclDof = 1;
                    MasterLclDof = [2 3]; %master is dof2 dof3
                    
                elseif (abs(dRdu2) >=  abs(dRdu1)) && (abs(dRdu2) >=  abs(dRdu3))
                    SlaveLclDof = 2;
                    MasterLclDof = [1 3]; %master is dof1 dof3
                elseif (abs(dRdu3) >=  abs(dRdu1)) && (abs(dRdu3) >=  abs(dRdu2))
                    SlaveLclDof = 3;
                    MasterLclDof = [1 2]; %master is dof2 dof3
                end
                
                MasterDof = dof(1,MasterLclDof); %master is dof1 dof3
                SlaveDof = dof(1,SlaveLclDof);
                
                dRds = dRdu(1,SlaveLclDof);
                dRdm1 = dRdu(1,MasterLclDof(1,1));
                dRdm2 = dRdu(1,MasterLclDof(1,2));
                
                Pmatlocal = [-dRdm1/dRds, -dRdm2/dRds];
                
                dsdm1 = Pmatlocal(1,1);
                dsdm2 = Pmatlocal(1,2);
                
                ddRdsds = ddRddu(SlaveLclDof,SlaveLclDof);
                ddRdsdm1 = ddRddu(SlaveLclDof,MasterLclDof(1,1));
                ddRdsdm2 = ddRddu(SlaveLclDof,MasterLclDof(1,2));
                ddRdm1ds = ddRddu(MasterLclDof(1,1),SlaveLclDof);
                ddRdm1dm1 = ddRddu(MasterLclDof(1,1),MasterLclDof(1,1));
                ddRdm1dm2 = ddRddu(MasterLclDof(1,1),MasterLclDof(1,2));
                ddRdm2ds = ddRddu(MasterLclDof(1,2),SlaveLclDof);
                ddRdm2dm1 = ddRddu(MasterLclDof(1,2),MasterLclDof(1,1));
                ddRdm2dm2 = ddRddu(MasterLclDof(1,2),MasterLclDof(1,2));
                
                % dd1d2d2 = -(ddRdu2du2*dRdu1 - dRdu2*ddRdu1du2)/(dRdu1^2);
                % dd1d2d3 = -(ddRdu2du3*dRdu1 - dRdu2*ddRdu1du3)/(dRdu1^2);
                % d1d3d2 = -(ddRdu3du2*dRdu1 - dRdu3*ddRdu1du2)/(dRdu1^2);
                % dd1d3d3 = -(ddRdu3du3*dRdu1 - dRdu3*ddRdu1du3)/(dRdu1^2);
                
                ddsdm1dm1 = -((ddRdm1ds*dsdm1+ddRdm1dm1)*dRds - dRdm1*(ddRdsds*dsdm1+ddRdsdm1))/(dRds^2);
                ddsdm1dm2 = -((ddRdm1ds*dsdm2+ddRdm1dm2)*dRds - dRdm1*(ddRdsds*dsdm2+ddRdsdm2))/(dRds^2);
                ddsdm2dm1 = -((ddRdm2ds*dsdm1+ddRdm2dm1)*dRds - dRdm2*(ddRdsds*dsdm1+ddRdsdm1))/(dRds^2);
                ddsdm2dm2 = -((ddRdm2ds*dsdm2+ddRdm2dm2)*dRds - dRdm2*(ddRdsds*dsdm2+ddRdsdm2))/(dRds^2);
                
                % ddsdm1dm1 = -((ddRdm1ds*dsdm1+ddRdm1dm1 )*dRds - dRdm1*(ddRdsds*dsdm1 + ddRdsdm1))/(dRds^2);
                % ddsdm1dm2 = -((ddRdm1ds*dsdm2+ddRdm1dm2)*dRds - dRdm1*(ddRdsds*dsdm2+ddRdsdm2))/(dRds^2);
                % ddsdm2dm1 = -((ddRdm2ds*dsdm2+ddRdm2dm1)*dRds - dRdm2*(ddRdsds*dsdm1+ddRdsdm1))/(dRds^2);
                % ddsdm2dm2 = -((ddRdm2ds*dsdm2+ddRdm2dm2)*dRds - dRdm2*(ddRdsds*dsdm2+ddRdsdm2))/(dRds^2);
                
                dPmatlocal = [ddsdm1dm1 ddsdm1dm2;
                    ddsdm2dm1 ddsdm2dm2];
                
                Pmatlocal22 = fFunc.dFelimActual(Point2B,SlaveLclDof);
                dPmatlocal22 = fFunc.ddFelimActual(Point2B,SlaveLclDof);
                if norm(Pmatlocal - Pmatlocal22)> 1e-9 %|| norm(dPmatlocal-dPmatlocal22)> 1e-9
                    %fprintf('fail')
                    Failcount = 1;
                    
                end
                
                %                 fprintf('test');
            else %actual second derivitive
                if (abs(dRdu1) >=  abs(dRdu2)) && (abs(dRdu1) >=  abs(dRdu3))
                    sdof = 1;
                elseif (abs(dRdu2) >=  abs(dRdu1)) && (abs(dRdu2) >=  abs(dRdu3))
                    sdof = 2;
                elseif (abs(dRdu3) >=  abs(dRdu1)) && (abs(dRdu3) >=  abs(dRdu2))
                    sdof = 3;
                end
                SlaveDof = dof(1,sdof);
                MasterDof= dof(1, setdiff(1:3,sdof)); %master is dof2
                Pmatlocal = fFunc.dFelimActual(Point2B,sdof);
                dPmatlocal = fFunc.ddFelimActual(Point2B,sdof);
            end
            
            
            Slavedofs_temp(SlaveDof,1) = 1;
            Masterdofs_temp(MasterDof,1) = 1;
            Pmat_temp(SlaveDof,MasterDof) = Pmatlocal;
            dPdm_mat_temp(MasterDof,MasterDof) = dPmatlocal*F(SlaveDof,1);
        end
        
        SlaveDofs = find(Slavedofs_temp == 1);
        MasterDofs = find(Masterdofs_temp == 1);
        
        %Solve degrees of freedom
        solvedof = [fdof;
            MasterDofs];
        
        Pmat = Pmat_temp(SlaveDofs,MasterDofs);
        dPdm_mat_term2 = dPdm_mat_temp(MasterDofs,MasterDofs);
        
        RHS_f = F(fdof,1);
        RHS_ms = F(MasterDofs,1) + Pmat'*F(SlaveDofs,1);
        RHS = [RHS_f;
            RHS_ms];
        
        Kff = DFDU(fdof,fdof);
        Kfm = DFDU(fdof,MasterDofs) + DFDU(fdof,SlaveDofs)*Pmat;
        Kmf = DFDU(MasterDofs,fdof) + Pmat'*DFDU(SlaveDofs,fdof);
        Kmm = DFDU(MasterDofs,MasterDofs) + dPdm_mat_term2 +  Pmat'*DFDU(SlaveDofs,MasterDofs) + DFDU(MasterDofs,SlaveDofs)*Pmat + Pmat'*DFDU(SlaveDofs,SlaveDofs)*Pmat;
        
        %Elmination Method mpcs K_global
        K_global = [Kff Kfm;
            Kmf Kmm];
        
        % Newton update
        Supdate = zeros(ndof,1);
        
        tic; Supdate(solvedof,1) = -K_global\RHS;toc%% Lagrange for circle
     
%         AAA = diag(diag(K_global))*10;
%         tic; d = -K_global\RHS; toc
%         tic; e = -cgs(K_global,RHS,[],100);toc
%         tic; f = -cgs(AAA*K_global,AAA*RHS,[],100);toc
%        
%         tic; e = -pcg(K_global,RHS,[],100);toc
%         tic; f = -pcg(AAA*K_global,AAA*RHS,[],100);toc
%         
%         tic; Lchol = chol(K_global); aaa = -(Lchol'\(Lchol\RHS));toc;
%         norm(-K_global*Supdate(solvedof,1)-RHS)
%         tic; e = -cgs(K_global,RHS,[],100);toc
%         norm(K_global*e-RHS)

%         LSopts.POSDEF = false;
%         LSopts.SYM = false;
% %         tic; f = -linsolve(K_global, RHS, LSopts); toc
% %         norm(-K_global*f-RHS)
%         LSopts.SYM = true;
%         tic; g = -linsolve(K_global, RHS, LSopts); toc
%         norm(-K_global*g-RHS)
%         
%         clear LSopts; LSopts.UHESS = true;
%         tic; h = -linsolve(K_global, RHS, LSopts); toc
%         norm(-K_global*h-RHS)
%         
%         Supdate(solvedof,1) = g;
        
        maxchange = max(max(abs(Supdate(MasterDofs,1))))./0.1*h_max;
        ndof
        if maxchange > 1
            delta = 1/maxchange;
        else
            delta = 1;
        end
        Supdate(solvedof,1) = delta.*Supdate(solvedof,1);
        
        for i = MPCnums'
            mpccount = mpccount +1;
            
            dof = i*3-2:i*3;
            pcurold = pold(i,:);
            ulocold = u(dof);
            
            PointPrev = pcurold + ulocold';
            
            if sum(dof(1) == MasterDofs) == 0
                lclslavedof = 1;
                lclmasterdof = [2 3];
            elseif sum(dof(2) == MasterDofs) == 0
                lclslavedof = 2;
                lclmasterdof = [1 3];
            else
                lclslavedof = 3;
                lclmasterdof = [1 2];
            end
            
            SupdateSlave = Supdate(dof(1,lclmasterdof),1);
            SlaveUpdate = fFunc.FuncFelimActual(PointPrev,SupdateSlave,lclslavedof);
            Supdate(dof(1,lclslavedof),1) = real(SlaveUpdate);
            
        end
        
        
        %
        %          for i = MPCnums'
        % %             mpccount = mpccount +1;
        %
        %             dof = i*3-2:i*3;
        %             pcurold = pold(i,:);
        %             ulocold = u(dof);
        %
        %             PointPrev = pcurold + ulocold';
        %
        %             if sum(dof(1) == MasterDofs) == 0
        %                 lclslavedof = 1;
        %                 lclmasterdof = [2 3];
        %             elseif sum(dof(2) == MasterDofs) == 0
        %                 lclslavedof = 2;
        %                 lclmasterdof = [1 3];
        %             else
        %                 lclslavedof = 3;
        %                 lclmasterdof = [1 2];
        %             end
        %
        %
        %             %Update the slave usign the second order taylor
        %             Pmatlocal = fFunc.dFelimActual(PointPrev,lclslavedof);
        %             dPmatlocal = fFunc.ddFelimActual(PointPrev,lclslavedof);
        %
        %             SupdateMaster = Supdate(dof(lclmasterdof),1);
        %
        %             SlaveUpdate = Pmatlocal*SupdateMaster+ 0.5*SupdateMaster'*dPmatlocal*SupdateMaster;
        %             Supdate(dof(1,lclslavedof),1) = real(SlaveUpdate);
        %
        %             % Snap the points to the surface
        %         end
        
        
        
        
        
        
        % Reshape to same as p
        update = Supdate(1:ndof,1);%% Lagrange for circle
        P_update = [update(1:3:end,1), update(2:3:end,1), update(3:3:end,1)];
        
        % Determine proportion of update to be used
        % if any bar wants to change by more than half its length, then reduce
        % update size
        
        barvecupdate = P_update(bars(:,1),:)-P_update (bars(:,2),:);
        barchange = sqrt(sum(barvecupdate.^2,2));
        change = abs(barchange./L);
        maxchange = max(change);
        
        if maxchange > UpdateLimiter
            delta = UpdateLimiter/maxchange;
        else
            delta = 1;
        end
        
        u = u + delta*update;
        p = pold+[u(1:3:end),u(2:3:end),u(3:3:end)];
        
    end
    
    newtonsteps = newtonsteps +1;
    
    %% end CB2
    
    % 8. Termination criterion:
    if ploton ==  true
        %PlotMPCs points around edge
        hold on
        plot3(p(MPCnums,1),p(MPCnums,2),p(MPCnums,3),'r.')
        plot3(p(pnums,1),p(pnums,2),p(pnums,3),'k*')
        drawnow
    end
    %     axis([-3 3 -3 3])
    axis equal
    hold off
    
    %    if max(abs(update)) < endtol
    qual = simpqual(p,tet,1);
    minqual = min(qual);
    NOrmupdt = norm(update)
    
    if NOrmupdt < 1
        AllowNewts = 5;
        densityctrlfreq = 5;
    end
    
    if NOrmupdt < 0.01
        AllowNewts = 10;
        densityctrlfreq = 10;
    end
    
    if NOrmupdt < 0.001
        AllowNewts = 10;
        densityctrlfreq = 10;
        ForceNewts = true
    end
    %     if minqual > 0.6
    %         ForceNewts = true;
    %         types(find(types == typeENUM.mpc)) = typeENUM.fixed;
    %         %AllowNewts = 3;
    %         densityctrlfreq = 3;
    %     else
    %         %AllowNewts = 3;
    %         densityctrlfreq = 5;
    %     end
    
    counter2 = counter2 +1;
    upds(counter2,1) = NOrmupdt;
        figure(4); 
    cla; plot(log10(upds)); drawnow; 
    figure(1)
    if (NOrmupdt < endtol)
%         p = pold;save('test.mat','p','types')
        break;
    end
    
    
    figure(MPCType+2);
    cla; plot(log10(upds)); drawnow; 
    figure(1)
    %     if max(sqrt(sum(deltat*Ftot(d<-geps,:).^2,2))/h0)<dptol, break; end
end
% detailinf.NormUpdates = NormUpdates;
% detailinf.RemeshedAtIter = ReMeshedAtIter;
% detailinf.RemovedPsCount = removedpcount;
% detailinf.RemeshedTimes = RemeshTimer;
% detailinf.TrussContributionTimer = TrussContributionTimer;
% detailinf.MpcCounter = MpcCounter;
% detailinf.PCounter = PCounter;
% detailinf.SolveDofCounter = SolveDofCounter;
% 
% detailinf.MPCResolveCounter = MPCResolveCounter;
% detailinf.SystemSolveTimes = SystemSolveTimer;
% detailinf.MPCTimes = MpcContributionTimer;



%%
% % % %% Sensitivityies 1 finite difference
% % % sensitivityMat = zeros(size(p,1)*size(p,2),size(fFunc.Params,1));
% % % 
% % % MPCnums = find(types==typeENUM.mpc);
% % % MPCdofSEn = sort([MPCnums*3-2;MPCnums*3-1;MPCnums*3]);
% % % pnums = find(types==typeENUM.fixed);
% % % pdof = sort([pnums*3-2;pnums*3-1;pnums*3]);
% % % fnums = find(types==typeENUM.free);
% % % fdofSens = sort([fnums*3-2;fnums*3-1;fnums*3]);
% % % pfnums = [pnums; fnums];
% % % pfdof = [pdof;fdofSens];
% % % 
% % % 
% % % % use the central difference method for best results, it makes the
% % % % sensitivities calc more expensive, but improves the sensitivities
% % % % massively compare these to each other.
% % % 
% % % 
% % % % % Method1 boundary only solve
% % % % fdofSen = []; L0Sen = L0; MPConly = false; %boundary only Use central difference with the original lengths
% % % % fdofSen = [];  barvec=p(bars(:,1),:) - p(bars(:,2),:); L0Sen = 0.7*sqrt(sum(barvec.^2,2)); MPConly = false; %boundary only Use central difference with slightly stretched current lengths
% % % 
% % % % % Method2
% % % fdofSen = fdof; L0Sen = L0; MPConly = false; %Full system solve
% % % 
% % % % % Method3
% % % % fdofSen = []; barvec=p(bars(:,1),:) - p(bars(:,2),:); L0Sen = 1.05*sqrt(sum(barvec.^2,2)); MPConly = true; %Boundary contributions only
% % % 
% % % %Super fail
% % % % fdofSen = []; L0Sen = L0; MPConly = true; %Boundary contributions only with original lengths
% % % % fdofSen = []; barvec=p(bars(:,1),:) - p(bars(:,2),:); L2 = 1*sqrt(sum(barvec.^2,2)); L0Sen = hbars*sqrt(sum(L2.^2)/sum(hbars.^2)); MPConly = true; %Boundary contributions only with original lengths
% % % 
% % % Forig = F;
% % % 
% % % if 1==MPC_lagrange
% % %     Kfforig = K_global(fdofSens,fdofSens); %lagrangian
% % %     boundofs = [pdof; MPCdofSEn];
% % %     Kfborig = K_global(fdofSens,boundofs); %lagrangian
% % %     K_global_orig = K_global;
% % %     DFDU_orig = DFDU;
% % %     lambda_orig = lambda;
% % %     DRmpcdUc_orig = DRmpcdUc;
% % %     DDRmpcddUc_orig = DDRmpcddUc;
% % % else
% % %     MasterDofs_Orig = MasterDofs;
% % %     Slavedofs_Orig = SlaveDofs;
% % %     PDofsOrig = pdof;
% % %     Kfforig = Kff;
% % %     Kfmorig = Kfm;
% % %     Kfporig = DFDU(fdof,pdof);
% % %     K_global_orig = K_global;
% % %     Rms([MasterDofs; SlaveDofs],1) = -F([MasterDofs;SlaveDofs],1);
% % %     Pmat_orig = Pmat;
% % %     DFDU_orig = DFDU;
% % %     boundofs = [pdof; MPCdofSEn];
% % % end
% % % 
% % % disp('Numerical sens')
% % % 
% % % for Central = 1:1 %used for central difference 1 for forward difference 2 for central difference
% % %     for ParNum = 1:size(fFunc.Params,1)
% % %         pold = p;
% % %         deps = 1E-5;
% % %         depsVec = zeros(size(fFunc.Params,1),1);
% % %         depsVec(ParNum,1) = deps;
% % %         
% % %         lambdaSen = zeros(nmpc,1); %only used for la grange
% % %         ldofSen = ((ndof+1):(ndof+nmpc))';
% % %         
% % %         tmpFunc = fFunc;
% % %         tmpFunc.Params = fFunc.Params + (-1)^(Central-1)*depsVec;
% % %         
% % %         % number of mpc nodes lagrange
% % %         
% % %         nmpc = size(MPCnums,1);
% % %         
% % %         
% % %         %% pold(MPCnums,:) = pold(MPCnums,:)*1.01;
% % %         % find number of fixed nodes && freenodes
% % %         
% % %         % find number of free nodes
% % %         
% % %         ndof = size(MPCdofSEn,1)+size(pfdof,1); %truss contribution total dofs for solve here
% % %         
% % %         u = zeros(ndof,1);
% % %         
% % %         %Add the pfix updates wrt to params
% % %         lclfuncpfixs_orig = fFunc.pfix;
% % %         lclfuncpfixs_deps = tmpFunc.pfix;
% % %         for FixPnum = pnums'
% % %             lclP = p(FixPnum,:);
% % %             
% % %             % check if it was original pfix
% % %             for FuncPFixno = 1:size(lclfuncpfixs_orig,1)
% % %                 
% % %                 if norm(lclP-lclfuncpfixs_orig(FuncPFixno,:)) < 1e-8
% % %                     dof = sort([FixPnum*3-2;FixPnum*3-1;FixPnum*3]);
% % %                     u(dof) = lclfuncpfixs_deps(FuncPFixno,:) - lclfuncpfixs_orig(FuncPFixno,:);
% % %                 end
% % %             end
% % %         end
% % %         
% % %         remesh = false;
% % %         newtonsteps = 0;
% % %         MESHplotter3D(p,tet,MPCType)
% % %         newtIt = 0;
% % %         
% % %         
% % %         while true
% % %             newtIt = newtIt +1;
% % %             
% % %             %% Truss Consistant tangent contributions
% % %             F = zeros(ndof,1);
% % %             DFDU = zeros(ndof,ndof);
% % %             
% % %             for i = 1:size(bars,1)
% % %                 dof = [bars(i,1)*3-2, bars(i,1)*3-1,bars(i,1)*3-0, bars(i,2)*3-2, bars(i,2)*3-1, bars(i,2)*3-0]';
% % %                 L0bar = L0(i,1); %desired length of bar
% % %                 uloc = u(dof);
% % %                 x = [pold(bars(i,1),:), pold(bars(i,2),:)]';
% % %                 [dFdu,broken,Ft] = CB_3dtruss_CT(uloc,x,L0bar); % CB_2dtruss_CT has 3x methods for Bar contribution.
% % %                 
% % %                 if broken ==  true
% % %                     break
% % %                 end
% % %                 
% % %                 DFDU(dof,dof) = DFDU(dof,dof) + dFdu;
% % %                 
% % %                 F(dof) = F(dof) + Ft;
% % %             end
% % %     
% % %             
% % %             %% Truss Consistant tangent contributions
% % % %             F = zeros(ndof,1);
% % % %             DFDU = zeros(ndof,ndof);
% % % %             
% % % %             for i = 1:size(bars,1)
% % % %                 
% % % %                 %                 if ((any(MPCnums == bars(i,1)) || any(MPCnums == bars(i,2))) && MPConly == false) ||...
% % % %                 %                         ((any(MPCnums == bars(i,1)) || any(pnums == bars(i,1))) && (any(MPCnums == bars(i,1)) || any(pnums == bars(i,1))) && MPConly == true) ||...%assemble only bars attached to MPCnodes and not interior nodes
% % % %                 %                         (norm(fdofSen)> 0)
% % % %                 
% % % %                dof = [bars(i,1)*3-2, bars(i,1)*3-1,bars(i,1)*3-0, bars(i,2)*3-2, bars(i,2)*3-1, bars(i,2)*3-0]';
% % % %                 L0bar = L0Sen(i,1); %desired length of bar
% % % %                 uloc = u(dof);
% % % %                 x = [pold(bars(i,1),:), pold(bars(i,2),:)]';
% % % %                 [dFdu,broken,Ft] = CB_3dtruss_CT(uloc,x,L0bar); % CB_2dtruss_CT has 3x methods for Bar contribution.
% % % %                 
% % % %                 if broken ==  true
% % % %                     break
% % % %                 end
% % % %                 
% % % %                 DFDU(dof,dof) = DFDU(dof,dof) + dFdu;
% % % %                 
% % % %                 F(dof) = F(dof) + Ft;
% % % %                 %                 end
% % % %             end
% % % %             
% % %             %% MPC methods contributions
% % %             % These mehtods can be selected at the input parameters at the top of this script
% % %             % La grange and Ellimination methids have seperate newton step updates
% % %             
% % %             
% % %             if 1==MPC_lagrange %% Lagrange for circle
% % %                 mpccount = 0;
% % %                 DRmpcdUc = sparse(nmpc,ndof);
% % %                 DDRmpcddUc = sparse(ndof,ndof);
% % %                 RMpc = sparse(nmpc,1);
% % %                 
% % %                 
% % %                 for i = MPCnums'
% % %                     mpccount = mpccount +1;
% % %                     
% % %                     dof = i*3-2:i*3;
% % %                     
% % %                     pcur = pold(i,:);
% % %                     uloc = u(dof);
% % %                     
% % %                     Point2B = pcur + uloc';
% % %                     
% % %                     RMpc(mpccount,1) = tmpFunc.F(Point2B);
% % %                     
% % %                     DRmpcdUc(mpccount,dof) = tmpFunc.dF(Point2B);
% % %                     ddRddu = tmpFunc.ddF(Point2B);
% % %                     DDRmpcddUc(dof,dof) = DDRmpcddUc(dof,dof)+lambdaSen(mpccount,1)*ddRddu;
% % %                 end
% % %                 
% % %                 
% % %                 K_global2 = [(DFDU+DDRmpcddUc) DRmpcdUc'
% % %                     DRmpcdUc sparse(nmpc,nmpc)];
% % %                 
% % %                 RHS = sparse(ndof+nmpc,1);
% % %                 RHS(pdof,1) = F(pdof,1);
% % %                 RHS(fdofSen,1) = F(fdofSen,1);
% % %                 RHS(ldofSen,1) = RMpc;
% % %                 RHS(MPCdofSEn,1) = F(MPCdofSEn,1)+(DRmpcdUc(:,MPCdofSEn))'*lambdaSen;
% % %                 
% % %                 
% % %                 
% % %                 %solve dof
% % %                 solvedof = [fdofSen;MPCdofSEn;ldofSen];
% % %                 
% % %                 % Newton update
% % %                 Supdate = zeros(ndof+nmpc,1);
% % %                 Supdate(solvedof) = -K_global2(solvedof,solvedof)\RHS(solvedof);
% % %                 
% % %                 update = Supdate(1:ndof,1);
% % %                 delta_lam = Supdate(ldofSen,1);
% % %                 
% % %                 %     %Min Norm update attempt
% % %                 P_update = [update(1:3:end,1), update(2:3:end,1),update(3:3:end,1)];
% % %                 
% % %                 % Determine proportion of update to be used
% % %                 % if any bar wants to change by more than half its length, then reduce
% % %                 % update size
% % %                 
% % %                 barvecupdate = P_update(bars(:,1),:)-P_update (bars(:,2),:);
% % %                 barchange = sqrt(sum(barvecupdate.^2,2));
% % %                 change = abs(barchange./L);
% % %                 maxchange = max(change);
% % %                 
% % %                 if maxchange > UpdateLimiter
% % %                     delta = UpdateLimiter/maxchange;
% % %                 else
% % %                     delta = 1;
% % %                 end
% % %                 
% % %                 lambdaSen = lambdaSen + delta*delta_lam;
% % %                 u = u + delta*update;
% % %                 
% % %             else
% % %                 %% Elimination methods
% % %                 %  Selection is created is done by the slave selection as the partial
% % %                 %  derivitive with the greatest constribution.
% % %                 
% % %                 Slavedofs_temp = zeros(ndof,1);
% % %                 Masterdofs_temp = zeros(ndof,1);
% % %                 Pmat_temp = sparse(ndof,ndof);
% % %                 dPdm_mat_temp = zeros(ndof,ndof);
% % %                 mpccount = 0;
% % %                 for i = MPCnums'
% % %                     mpccount = mpccount +1;
% % %                     
% % %                     dof = i*3-2:i*3;
% % %                     
% % %                     pcur = pold(i,:);
% % %                     uloc = u(dof);
% % %                     
% % %                     Point2B = pcur + uloc';
% % %                     
% % %                     dRdu = tmpFunc.dF(Point2B);
% % %                     
% % %                     dRdu1 = dRdu(1,1);
% % %                     dRdu2 = dRdu(1,2);
% % %                     dRdu3 = dRdu(1,3);
% % %                     
% % %                     if 0 == EllimACt % CB2 second partial dirivitive approximation
% % %                         %                         ddRddu = fFunc.ddF(Point2B);
% % %                         %
% % %                         %                         ddRdu1du1 = ddRddu(1,1);
% % %                         %                         ddRdu1du2 = ddRddu(1,2);
% % %                         %                         ddRdu2du1 = ddRddu(2,1);
% % %                         %                         ddRdu2du2 = ddRddu(2,2);
% % %                         %
% % %                         %                         if any(MasterDofs_Orig == dof(1,2))
% % %                         %                             MasterDof = dof(1,2); %master is dof2
% % %                         %                             SlaveDof = dof(1,1);
% % %                         %                             Pmatlocal = -dRdu2/dRdu1;
% % %                         %                             %dPmatlocal = -(ddRdu2du2*dRdu1 - dRdu2*ddRdu1du2)/(dRdu1^2); %Incorrect
% % %                         %                             dPmatlocal = -((ddRdu2du1*Pmatlocal+ddRdu2du2)*dRdu1 - dRdu2*(ddRdu1du1*Pmatlocal+ddRdu1du2))/(dRdu1^2);
% % %                         %
% % %                         %                         elseif any(MasterDofs_Orig == dof(1,1))
% % %                         %                             MasterDof = dof(1,1); %master is dof1
% % %                         %                             SlaveDof = dof(1,2);
% % %                         %                             Pmatlocal = -dRdu1/dRdu2;
% % %                         %                             %dPmatlocal = -(ddRdu1du1*dRdu2 - dRdu1*ddRdu2du1)/dRdu2^2; %incorrect
% % %                         %                             dPmatlocal = -((ddRdu1du2*Pmatlocal +  ddRdu1du1)*dRdu2 - dRdu1*(ddRdu2du2*Pmatlocal+ddRdu2du1))/dRdu2^2; %Campbell
% % %                         %                         end
% % %                         
% % %                     else %actual second derivitive
% % %                         
% % %                         if sum(dof(1) == MasterDofs) == 0
% % %                             sdof = 1;                            
% % %                         elseif sum(dof(2) == MasterDofs) == 0
% % %                             sdof = 2;                            
% % %                         else
% % %                             sdof = 3;                            
% % %                         end
% % %                         
% % %                         SlaveDof = dof(1,sdof);
% % %                         MasterDof= dof(1, setdiff(1:3,sdof)); %master is dof2
% % %                         Pmatlocal = tmpFunc.dFelimActual(Point2B,sdof);
% % %                         dPmatlocal = tmpFunc.ddFelimActual(Point2B,sdof);
% % %                     end
% % %             
% % %                     Slavedofs_temp(SlaveDof,1) = 1;
% % %                     Masterdofs_temp(MasterDof,1) = 1;
% % %                     Pmat_temp(SlaveDof,MasterDof) = Pmatlocal;
% % %                     dPdm_mat_temp(MasterDof,MasterDof) = dPmatlocal*F(SlaveDof,1);
% % %                     
% % %                 end
% % %                
% % %                 SlaveDofs = find(Slavedofs_temp == 1);
% % %                 MasterDofs = find(Masterdofs_temp == 1);
% % %                 
% % %                 %Solve degrees of freedom
% % %                 solvedof = [fdof;
% % %                     MasterDofs];
% % %                 
% % %                 Pmat = Pmat_temp(SlaveDofs,MasterDofs);
% % %                 dPdm_mat_term2 = dPdm_mat_temp(MasterDofs,MasterDofs);
% % %                 
% % %                 RHS_f = F(fdof,1);
% % %                 RHS_ms = F(MasterDofs,1) + Pmat'*F(SlaveDofs,1);
% % %                 RHS = [RHS_f;
% % %                     RHS_ms];
% % %                 
% % %                 Kff = DFDU(fdof,fdof);
% % %                 Kfm = DFDU(fdof,MasterDofs) + DFDU(fdof,SlaveDofs)*Pmat;
% % %                 Kmf = DFDU(MasterDofs,fdof) + Pmat'*DFDU(SlaveDofs,fdof);
% % %                 Kmm = DFDU(MasterDofs,MasterDofs) + dPdm_mat_term2 +  Pmat'*DFDU(SlaveDofs,MasterDofs) + DFDU(MasterDofs,SlaveDofs)*Pmat + Pmat'*DFDU(SlaveDofs,SlaveDofs)*Pmat;
% % %                 
% % %                 %Elmination Method mpcs K_global
% % %                 K_global2 = [Kff Kfm;
% % %                     Kmf Kmm];
% % %                 
% % %                 % Newton update
% % %                 Supdate = zeros(ndof,1);
% % %                 Supdate(solvedof,1) = -K_global2\RHS;%% Lagrange for circle
% % %                 
% % %                 maxchange = max(max(abs(Supdate(MasterDofs,1))))./0.1*h_max;
% % %         
% % %                 if maxchange > 1
% % %                     delta = 1/maxchange;
% % %                 else
% % %                     delta = 1;
% % %                 end
% % %                 Supdate(solvedof,1) = delta.*Supdate(solvedof,1);
% % %                 
% % %                 for i = MPCnums'
% % %                     mpccount = mpccount +1;
% % %                     
% % %                     dof = i*3-2:i*3;
% % %                     pcurold = pold(i,:);
% % %                     ulocold = u(dof);
% % %                     
% % %                     PointPrev = pcurold + ulocold';
% % %                     
% % %                     if sum(dof(1) == MasterDofs) == 0
% % %                         lclslavedof = 1;
% % %                         lclmasterdof = [2 3];
% % %                     elseif sum(dof(2) == MasterDofs) == 0
% % %                         lclslavedof = 2;
% % %                         lclmasterdof = [1 3];
% % %                     else
% % %                         lclslavedof = 3;
% % %                         lclmasterdof = [1 2];
% % %                     end
% % %                     
% % %                     SupdateSlave = Supdate(dof(1,lclmasterdof),1);
% % %                     SlaveUpdate = tmpFunc.FuncFelimActual(PointPrev,SupdateSlave,lclslavedof);
% % %                     Supdate(dof(1,lclslavedof),1) = real(SlaveUpdate);
% % %                     
% % %                 end
% % %                 
% % %                 % Reshape to same as p
% % %                 update = Supdate(1:ndof,1);%% Lagrange for circle
% % %                 P_update = [update(1:3:end,1), update(2:3:end,1), update(3:3:end,1)];
% % %                 
% % %                 % Determine proportion of update to be used
% % %                 % if any bar wants to change by more than half its length, then reduce
% % %                 % update size
% % %                 
% % %                 barvecupdate = P_update(bars(:,1),:)-P_update (bars(:,2),:);
% % %                 barchange = sqrt(sum(barvecupdate.^2,2));
% % %                 change = abs(barchange./L);
% % %                 maxchange = max(change);
% % %                 
% % %                 if maxchange > UpdateLimiter
% % %                     delta = UpdateLimiter/maxchange;
% % %                 else
% % %                     delta = 1;
% % %                 end
% % %                 
% % %                 u = u + delta*update;
% % %                 
% % %             end
% % %             
% % %             NOrmupdt = norm(update)
% % %             
% % %             if NOrmupdt < 10e-8 && newtIt > 0
% % %                 if norm(fdofSen) == 0 %For boundary only solves this is true
% % %                     u(fdofSens,1) = -DFDU_orig(fdofSens,fdofSens)\(DFDU_orig(fdofSens,boundofs)*u(boundofs)); %sensitivities
% % %                 end
% % %                 break
% % %             end
% % %             
% % %             ps2 = pold+[u(1:3:end),u(2:3:end),u(3:3:end)];
% % %             if ploton == true
% % %                 %PlotMPCs points around edge
% % %                 hold on
% % %                 plot(ps2(MPCnums,1),ps2(MPCnums,2),'b.')
% % %                 plot(ps2(pfnums,1),ps2(pfnums,2),'g*')
% % %                 drawnow
% % %             end
% % %             
% % %         end
% % %         
% % %         if Central == 1
% % %             sensitivityMat(:,ParNum) = sensitivityMat(:,ParNum)+ u./deps;
% % %         elseif Central == 2
% % %             sensitivityMat(:,ParNum) = 0.5*sensitivityMat(:,ParNum)+(-1)^(Central-1)*0.5*u./deps;
% % %         end
% % %         
% % %     end
% % % end
% % % %%
% % % dp = 0.3;
% % % 
% % % xyorig = fFunc.plot;
% % % change = [-dp dp]';
% % % plotno = MPCType*10 +1;
% % % 
% % % 
% % % changes = [0.2  -0.2 -0.1
% % %     0  -dp 0];
% % % 
% % % plotno = MPCType*10;
% % % 
% % % for change = changes'
% % %     plotno = plotno+1;
% % %     UPDATER = sensitivityMat*change;
% % %     MESHplotter3D(p + [UPDATER(1:3:end,1), UPDATER(2:3:end,1), UPDATER(3:3:end,1)] ,tet,plotno)
% % %     tmpfunc = fFunc;tmpfunc.Params = tmpfunc.Params + change;
% % %     xy = tmpfunc.plot;
% % %     hold on;
% % %     plot(xy(:,1),xy(:,2),'r-','LineWidth',1.5)
% % %     plot(xyorig(:,1),xyorig(:,2),'b-');
% % %     %     EdgePlot(p,tet);
% % %     legend({'0.8 <= Qual','0.4 <= Qual < 0.8','Qual < 0.4','New Boundary','Original Boundary','Original Mesh'},'Location','northeastoutside')
% % %     break
% % % end
% % % 
% % % 
% % % 
% % % %% Sensitivity 2 analytical
% % % sensitivityMat2 = zeros(size(p,1)*size(p,2),size(fFunc.Params,1));
% % % 
% % % % use the central difference method for best results, it makes the
% % % % sensitivities calc more expensive, but improves the sensitivities
% % % % massively compare these to each other.
% % % 
% % % 
% % % % % Method2
% % % fdofSen = fdof; L0Sen = L0; %Full system solve
% % % 
% % % disp('Analytical sens')
% % % for ParNum = 1:size(fFunc.Params,1)
% % %     pold = p;
% % %     depsVec = zeros(size(fFunc.Params,1),1);
% % %     depsVec(ParNum,1) = deps;
% % %     
% % %     lambdaSen = zeros(nmpc,1); %only used for la grange
% % %     ldofSen = ((ndof+1):(ndof+nmpc))';
% % %     
% % %     tmpFunc = fFunc;
% % %     tmpFunc.Params = fFunc.Params + depsVec;
% % %     
% % %     % number of mpc nodes lagrange
% % %     
% % %     nmpc = size(MPCnums,1);
% % %     
% % %     % find number of fixed nodes && freenodes
% % %     
% % %     % find number of free nodes
% % %     
% % %     ndof = size(MPCdofSEn,1)+size(pfdof,1); %truss contribution total dofs for solve here
% % %     clear u
% % %     uSen = zeros(ndof,1);
% % %     %Add the pfix updates wrt to params
% % %     lclfuncpfixs_orig = fFunc.pfix;
% % %     lclfuncpfixs_deps = tmpFunc.pfix;
% % %     for FixPnum = pnums'
% % %         lclP = p(FixPnum,:);
% % %         
% % %         % check if it was original pfix
% % %         for FuncPFixno = 1:size(lclfuncpfixs_orig,1)
% % %             
% % %             if norm(lclP-lclfuncpfixs_orig(FuncPFixno,:)) < 1e-8
% % %                 dof = sort([FixPnum*3-2;FixPnum*3-1;FixPnum*3]);
% % %                 uSen(dof) = lclfuncpfixs_deps(FuncPFixno,:) - lclfuncpfixs_orig(FuncPFixno,:);
% % %             end
% % %         end
% % %     end
% % %     
% % %     remesh = false;
% % %     newtonsteps = 0;
% % %     MESHplotter3D(p,tet,MPCType)
% % %     newtIt = 0;
% % %     
% % %     
% % %     
% % %     
% % %     while true
% % %         newtIt = newtIt +1;
% % %         %% Truss Consistant tangent contributions
% % %         F_base = zeros(ndof,1);
% % %         F_peturb = zeros(ndof,1);
% % %         
% % %         for i = 1:size(bars,1)
% % %             
% % %             
% % %             dof = [bars(i,1)*3-2, bars(i,1)*3-1,bars(i,1)*3-0, bars(i,2)*3-2, bars(i,2)*3-1, bars(i,2)*3-0]';
% % %             
% % %             L0bar = L0Sen(i,1); %desired length of bar
% % %             uloc = uSen(dof);
% % %             x = [p(bars(i,1),:), p(bars(i,2),:)]';
% % %             [dFdu,broken,Ft_base] = CB_3dtruss_CT(0*uloc,x,L0bar); % base truss force matix
% % %             [dFdu,broken,Ft_peturb] = CB_3dtruss_CT(uloc,x,L0bar);% CB_2dtruss_CT has 3x methods for Bar contribution.
% % %             
% % %             if broken == true
% % %                 break
% % %             end
% % %             
% % %             %             DFDUsen(dof,dof) = DFDUsen(dof,dof) + dFdu;
% % %             %
% % %             F_base(dof) = F_base(dof) + Ft_base;
% % %             F_peturb(dof) = F_peturb(dof) + Ft_peturb;
% % %             
% % %         end
% % %         dF =  F_peturb-F_base;
% % %         %% MPC methods contributions
% % %         % These mehtods can be selected at the input parameters at the top of this script
% % %         % La grange and Ellimination methids have seperate newton step updates
% % %         
% % %         if 1==MPC_lagrange %% Lagrange for circle
% % %             mpccount = 0;
% % %             DRmpcdUc = sparse(nmpc,ndof);
% % %             DDRmpcddUc = sparse(ndof,ndof);
% % %             RMpc_base = sparse(nmpc,1);
% % %             RMpc_peturb = sparse(nmpc,1);
% % %             
% % %             
% % %             
% % %             for i = MPCnums'
% % %                 mpccount = mpccount +1;
% % %                 dof = i*3-2:i*3;
% % %                 pcur = pold(i,:);
% % %                 uloc = uSen(dof);
% % %                 
% % %                 Point2B = pcur + uloc';
% % %                 RMpc_base(mpccount,1) = fFunc.F(pcur); %
% % %                 RMpc_peturb(mpccount,1) = tmpFunc.F(Point2B);
% % %                 
% % %                 DRmpcdUc(mpccount,dof) = tmpFunc.dF(Point2B);
% % %                 ddRddu = tmpFunc.ddF(Point2B);
% % %             end
% % %             
% % %             dRHS = sparse(ndof+nmpc,1);
% % %             dRHS(ldofSen,1) = RMpc_peturb-RMpc_base;
% % %             dRHS(MPCdofSEn,1) = (DRmpcdUc(:,MPCdofSEn))'*lambda_orig;
% % %             
% % %             %solve dof
% % %             solvedof = [fdofSen;MPCdofSEn;ldofSen];
% % %             
% % %             % Newton update
% % %             dSupdate = zeros(ndof+nmpc,1);
% % %             dSupdate(solvedof) = K_global_orig(solvedof,solvedof)\(-dRHS(solvedof)-K_global_orig(solvedof,pdof)*uSen(pdof)); % note change due to sensitivity cal supdate no longer nega tive
% % %             
% % %             update = dSupdate(1:ndof,1);
% % %             delta_lam = dSupdate(ldofSen,1);
% % %             
% % %             %     %Min Norm update attempt
% % %             P_update = [update(1:3:end,1), update(2:3:end,1),update(3:3:end,1)];
% % %             
% % %             % Determine proportion of update to be used
% % %             % if any bar wants to change by more than half its length, then reduce
% % %             % update size
% % %             
% % %             lambdaSen = lambdaSen + delta_lam;
% % %             uSen = uSen  +update; %-usen for the addition of the prescribded dofs in the correct place
% % %             
% % %             
% % %             
% % %         else
% % %             %% Elimination methods
% % %             %  Selection is created is done by the slave selection as the partial
% % %             %  derivitive with the greatest constribution.
% % %             
% % %             Slavedofs_temp = zeros(ndof,1);
% % %             Masterdofs_temp = zeros(ndof,1);
% % %             Pmat_peturb_temp = zeros(ndof,ndof);
% % %             dPdm_mat_temp = zeros(ndof,ndof);
% % %             mpccount = 0;
% % %             for i = MPCnums'
% % %                 mpccount = mpccount +1;
% % %                 
% % %                 dof = i*3-2:i*3;
% % %                 
% % %                 pcur = pold(i,:);
% % %                 uloc = uSen(dof);
% % %                 
% % %                 Point2B = pcur + uloc';
% % %                 
% % %                 dRdU = tmpFunc.dF(Point2B);
% % %                 
% % %                 dRdu1 = dRdU(1,1);
% % %                 dRdu2 = dRdU(1,2);
% % %                 
% % %                 if 0 == EllimACt % CB2 second partial dirivitive approximation
% % %                                        
% % %                 else %actual second derivitive
% % %                     
% % %                     if sum(dof(1) == MasterDofs) == 0
% % %                         sdof = 1;
% % %                     elseif sum(dof(2) == MasterDofs) == 0
% % %                         sdof = 2;
% % %                     else
% % %                         sdof = 3;
% % %                     end
% % %                     
% % %                     SlaveDof = dof(1,sdof);
% % %                     MasterDof= dof(1, setdiff(1:3,sdof)); %master is dof2
% % %                     Pmatlocal = tmpFunc.dFelimActual(Point2B,sdof);
% % %                     dPmatlocal = tmpFunc.ddFelimActual(Point2B,sdof);
% % %                     D = tmpFunc.FuncFelimActual(Point2B,[0 0]',sdof);
% % %                                         
% % % %                     if any(MasterDofs_Orig == dof(1,2))
% % % %                         MasterDof = dof(1,2); %master is dof2
% % % %                         SlaveDof = dof(1,1);
% % % %                         %              Pmatlocal = -dRdu2/dRdu1;
% % % %                         Pmatlocal = tmpFunc.dFelimActual(Point2B,1);
% % % %                         dPmatlocal = tmpFunc.ddFelimActual(Point2B,1);
% % % %                         D = tmpFunc.FuncFelimActual(Point2B,0,2);
% % % %                     elseif any(MasterDofs_Orig == dof(1,1))
% % % %                         MasterDof = dof(1,1); %master is dof1
% % % %                         SlaveDof = dof(1,2);
% % % %                         %              Pmatlocal = -dRdu1/dRdu2;
% % % %                         Pmatlocal = tmpFunc.dFelimActual(Point2B,2);
% % % %                         dPmatlocal = tmpFunc.ddFelimActual(Point2B,2);
% % % %                         D = tmpFunc.FuncFelimActual(Point2B,0,1);
% % % %                     end
% % %                 end
% % %                 
% % %                 Slavedofs_temp(SlaveDof,1) = 1;
% % %                 Masterdofs_temp(MasterDof,1) = 1;
% % %                 Pmat_peturb_temp(SlaveDof,MasterDof) = Pmatlocal;
% % %                 dPdm_mat_temp(MasterDof,MasterDof) = dPmatlocal*F(SlaveDof,1);
% % %                 dPdm_mat_Only(MasterDof,MasterDof) = dPmatlocal;
% % %                 dd(SlaveDof,1) = D;
% % %             end
% % %             
% % %             SlaveDofs = find(Slavedofs_temp==1);
% % %             MasterDofs = find(Masterdofs_temp==1);
% % %             
% % %             %Solve degrees of freedom
% % %             solvedof = [fdofSen;
% % %                 MasterDofs];
% % %             Rm_in = Rms(MasterDofs);
% % %             Rs_in = Rms(SlaveDofs);
% % %             
% % %             Pmat = Pmat_peturb_temp(SlaveDofs,MasterDofs);
% % %             %                 Pmat = Pmat_orig;
% % %             dPdm_mat_term2 = dPdm_mat_temp(MasterDofs,MasterDofs);
% % %             
% % %             RHS_f =-DFDU_orig(fdofSen,SlaveDofs)*dd(SlaveDofs,1) - DFDU_orig(fdofSen,pdof)*uSen(pdof);
% % %             RHS_ms =  (Rm_in+Pmat'*Rs_in)...
% % %                 - DFDU_orig(MasterDofs,pdof)*uSen(pdof) - Pmat'*DFDU_orig(SlaveDofs,pdof)*uSen(pdof)... prescribed contributions
% % %                 -Pmat'*DFDU_orig(SlaveDofs,SlaveDofs)*dd(SlaveDofs,1)...
% % %                 -DFDU_orig(MasterDofs,SlaveDofs)*dd(SlaveDofs,1);
% % %             
% % %             RHS = [RHS_f;
% % %                    RHS_ms];
% % %             
% % %             % Newton update
% % %             Supdate = zeros(ndof,1);
% % %             Supdate(solvedof,1) = K_global_orig\(RHS);%% Lagrange for circle
% % %             
% % %             % Obtain the slave dofs update
% % %             
% % %                         Supdate(SlaveDofs,1) = Pmat*Supdate(MasterDofs,1) + dd(SlaveDofs);% optional +0.5*Supdate(MasterDofs,1)'*dPdm_mat_Only(MasterDofs,MasterDofs)*Supdate(MasterDofs,1);
% % %                         update = Supdate(1:ndof,1);
% % %             
% % % %             for i = MPCnums'
% % % %                 mpccount = mpccount +1;
% % % %                 
% % % %                 dof = i*3-2:i*3;
% % % %                 pcurold = pold(i,:);
% % % %                 ulocold = uSen(dof);
% % % %                 
% % % %                 PointPrev = pcurold + ulocold';
% % % %                 
% % % %                 if sum(dof(1) == MasterDofs) == 0
% % % %                     lclslavedof = 1;
% % % %                     lclmasterdof = [2 3];
% % % %                 elseif sum(dof(2) == MasterDofs) == 0
% % % %                     lclslavedof = 2;
% % % %                     lclmasterdof = [1 3];
% % % %                 else
% % % %                     lclslavedof = 3;
% % % %                     lclmasterdof = [1 2];
% % % %                 end
% % % %                 
% % % %                 SupdateSlave = Supdate(dof(1,lclmasterdof),1);
% % % %                 SlaveUpdate = tmpFunc.FuncFelimActual(PointPrev,SupdateSlave,lclslavedof);
% % % %                 Supdate(dof(1,lclslavedof),1) = real(SlaveUpdate);
% % % %                 
% % % %             end
% % % %             update = Supdate(1:ndof,1);
% % %             
% % %             
% % %             
% % %             %     %Min Norm update attempt
% % %             P_update = [update(1:3:end,1), update(2:3:end,1), update(3:3:end,1)];
% % %             
% % %             % Determine proportion of update to be used
% % %             % if any bar wants to change by more than half its length, then reduce
% % %             % update size
% % %             
% % %             barvecupdate = P_update(bars(:,1),:)-P_update (bars(:,2),:);
% % %             barchange = sqrt(sum(barvecupdate.^2,2));
% % %             change = abs(barchange./L);
% % %             maxchange = max(change);
% % %             
% % %             if maxchange > UpdateLimiter
% % %                 delta = UpdateLimiter/maxchange;
% % %             else
% % %                 delta = 1;
% % %             end
% % %             
% % %             uSen = uSen + update;
% % %             dFout = DFDU_orig*uSen;
% % %             Rm_out = dFout(MasterDofs,1);
% % %             Rs_out = dFout(SlaveDofs,1);
% % %         end
% % %         
% % %         
% % %         
% % %         NOrmupdt = norm(update)
% % %         
% % %         
% % %         ps2 = pold+[uSen(1:3:end),uSen(2:3:end),uSen(3:3:end)];
% % %         if ploton == true
% % %             %PlotMPCs points around edge
% % %             hold on
% % %             plot(ps2(MPCnums,1),ps2(MPCnums,2),'b.')
% % %             plot(ps2(pfnums,1),ps2(pfnums,2),'g*')
% % %             drawnow
% % %         end
% % %         
% % %         break
% % %     end
% % %     
% % %     
% % %     sensitivityMat2(:,ParNum) = uSen./deps;
% % %     
% % %     
% % % end
% % % 
% % % %%
% % % % dp = 0.3;
% % % 
% % % % xyorig = fFunc.plot;
% % % 
% % % plotno = MPCType*10 +1;
% % % 
% % % 
% % % % changes = [-dp  0
% % % %     0  -dp
% % % %     dp  0
% % % %     0   dp
% % % %     -dp -dp
% % % %     dp  dp
% % % %     -dp  dp
% % % %     dp -dp];
% % % 
% % % plotno = MPCType*10+20;
% % % 
% % % for change = changes'
% % %     plotno = plotno+1;
% % %     UPDATER = sensitivityMat2*change;
% % %     MESHplotter3D(p + [UPDATER(1:3:end,1), UPDATER(2:3:end,1), UPDATER(3:3:end,1)] ,tet,plotno)
% % %     t = surftri(p,tet);
% % %     s2 = patch('vertices',p,'faces',t,'edgecol','k','facecol','r'); drawnow;
% % %     s2.FaceAlpha = 0.5;
% % %     xlabel('x_1')
% % %     ylabel('x_2')
% % %     zlabel('x_3')
% % %     
% % %     tmpfunc = fFunc;tmpfunc.Params = tmpfunc.Params + change;
% % %     xy = tmpfunc.plot;
% % %     hold on;
% % %     plot(xy(:,1),xy(:,2),'r-','LineWidth',1.5)
% % %     plot(xyorig(:,1),xyorig(:,2),'b-');
% % %     %     EdgePlot(p,tet);
% % %     legend({'0.8 <= Qual','0.4 <= Qual < 0.8','Qual < 0.4','New Boundary','Original Boundary','Original Mesh'},'Location','northeastoutside')
% % %     break
% % % end
% % % 
% % % 
% % % 
% % % 
% % % %% End
% % % totalsen = [sensitivityMat sensitivityMat2];
% % % 
% % % sensNorms(1,1) = norm(sensitivityMat(:,1)-sensitivityMat2(:,1));
% % % sensNorms(2,1) = norm(sensitivityMat(:,2)-sensitivityMat2(:,2));
% % % sensNorms(3,1) = norm(sensitivityMat(:,3)-sensitivityMat2(:,3))
% % % 
% % % sensAves(1,1) = mean(abs(sensitivityMat(:,1)-sensitivityMat2(:,1)));
% % % sensAves(2,1) = mean(abs(sensitivityMat(:,2)-sensitivityMat2(:,2)));
% % % sensAves(3,1) = mean(abs(sensitivityMat(:,3)-sensitivityMat2(:,3)))
% % % 
% % % MESHplotter3D(p,tet,MPCType)
% % % 
minqual = min(qual)

npsend = size(p,1)
removedpcount
addedpcount
iters  = counter

lr = L./L0act;

minls = min(lr)
maxls = max(lr)
avelr = mean(lr)
time = toc
% % % 
% % % % pause
% % % 
% % % % Clean up and plot final mesh
% % % % % % [p,t] = fixmesh(p,t);
% % % % % % simpplot(p,t)
end

function [a] = MESHplotter3D(p,tet,fignum)
figure(fignum); cla
% simpplot(p,tet,'p(:,2)>0');
% title(['Retriangulation #'])
%         drawnow
t = unique(sort([tet(:,[1,2,3]);tet(:,[1,2,4]);tet(:,[1,3,4]);tet(:,[2,3,4]);],2),'rows');
cla;
s1 = patch('vertices',p,'faces',t,'edgecol','k','facecol',[.8,.9,1]);
s1.FaceAlpha = 1.00;
axis equal

% draw low quality elements
qual = simpqual(p,tet,1);
AD = find(qual<.2);

t = unique(sort([tet(AD,[1,2,3]);tet(AD,[1,2,4]);tet(AD,[1,3,4]);tet(AD,[2,3,4]);],2),'rows');

% hold on
% s2 = patch('vertices',p,'faces',t,'edgecol','k','facecol','r'); drawnow;
% s2.FaceAlpha = 0.5;
% hold off
end


