function [p,t,iters] = CB_distmesh2d_mpc(fFunc,fH,h_max,h_min,bbox,pfix,pmpc,endtol,MPCType,varargin)
%DISTMESH2D 2-D Mesh Generator using Distance Functions.
%   [P,T]=CB_distmesh2d_mpc(fFunc,fH,h_max,h_min,bbox,pfix,pmpc,endtol,varargin
%
%      P:         Node positions (Nx2)
%      T:         Triangle indices (NTx3)
%      fFunc:     Distance function d(x,y)
%      FH:        Scaled edge length function h(x,y)
%      h_max:     MaxEdge length(associated fH function)
%      h_min:     MaxEdge length(associated fH function)
%      BBOX:      Bounding box [xmin,ymin; xmax,ymax]
%      PFIX:      Fixed node positions (NFIXx2)
%      PMPC:      Any initiated MPC points
%      Endtol:    Termination tolerence for Newton step

%   See also: MESHDEMO2D, DISTMESHND, DELAUNAYN, TRIMESH.

%   distmesh2d.m v1.1
%   Copyright (C) 2004-2012 Per-Olof Persson. See COPYRIGHT.TXT for details.
tic

%% Set up parameters
ttol = 0.1; % Change tolerence for p movement
geps = 0.1;  % This the boudary limit for eliminating points
densityctrlfreq = 3; % density control frequency
destL = 1.5; % Length factor for length control on elements
UpdateLimiter = 20; % p update limiter This enforces stability for highly tensioned meshes
AllowNewts = 2; % Newtons teps allowed before remeshing
ploton = true; %Plot during evaluations (see mesh updates

%% MPC Method selection

if MPCType == 1 %La grange
    MPC_lagrange = 1; % 1 use la grange MPC 0 use ellimination
    EllimACt = 0; % Actual 1 approx 0 for Elimination method only
elseif MPCType == 2 %Ellim actual
    MPC_lagrange = 0; % 1 use la grange MPC 0 use ellimination
    EllimACt = 1; % Actual 1 approx 0 for Elimination method only
else %Ellim approx
    MPC_lagrange = 0; % 1 use la grange MPC 0 use ellimination
    EllimACt = 0; % Actual 1 approx 0 for Elimination method only
end

%% 1. Create initial distribution in bounding box (equilateral triangles)

h02 = 0.8*(0.1*h_max + .9*h_min); %initial mesh seed
h02 = 0.9*h_max;

[x,y]=meshgrid(bbox(1,1):h02:bbox(2,1),bbox(1,2):h02*sqrt(3)/2:bbox(2,2));
x(2:2:end,:) = x(2:2:end,:) + h02/2;                      % Shift even rows
p=[x(:),y(:)];                                       % List of node coordinates

%% 2. Remove points outside the region, apply the rejection method

% p=p((fFunc.fd(p) < +geps.*fH(p)),:);

% % r0=1./feval(fH,p,varargin{:}).^2;                    % Probability to keep point
% % p=p(rand(size(p,1),1)<r0./max(r0),:);                % Rejection method

if ~isempty(pfix), p=setdiff(p,pfix,'rows'); end     % Remove duplicated nodes
if ~isempty(pmpc), p=setdiff(p,pmpc,'rows'); end     % Remove duplicated nodes cb2

pfix = unique(pfix,'rows'); nfix=size(pfix,1);

%% Find unique mpc points of the input
pmpc = [pfix; pmpc];
[ ~, IA ,~] =   unique(pmpc,'rows','first');
IA = sort(IA,'ascend');
pmpc = pmpc(IA);
pmpc = pmpc(nfix+1:end,:); nmpc = size(pmpc,1);

%% Enumeration types 0 free, 1 fixed, 2 mpc
s = 4;
typeENUM.fixed = 0+s;
typeENUM.mpc = 1+s;
typeENUM.free = 2+s;
typeENUM.forcesnap = -1;

% Ensure unique fixed nodes
p = [pfix; pmpc; p];                                       % Prepend fix points

[~, IA, ~] = unique(p,'rows','first');
IA = sort(IA,'ascend'); %keeps a stable order very important to ensure the types tie up with the points correctly;  Octave to replace MATLABS unique(p,'rows','stable');
p = p(IA,:);

FixMpcTypes = [typeENUM.fixed*ones(size(pfix,1),1); typeENUM.mpc*ones(size(pmpc,1),1)];
types = typeENUM.free*ones(size(p,1),1);
types(1:size(FixMpcTypes,1)) = FixMpcTypes;

% For first variabel initialisation
counter=0;
counter2 = 0;
pold=inf;
removedpcount = 0;
addedpcount = 0;
minqual = 0;
densitynewts = inf;
newtonsteps = inf;
ForcedRemesh = inf;
ForceNewts = false;
remesh = false;
while 1
    counter = counter +1;
    if counter == 11
       disp('10') 
    end
    if ForceNewts == false
        
        %% 3. Retriangulation by the Delaunay algorithm
        if ((max(sqrt(sum((p-pold).^2,2))./feval(fH,p)) > ttol) && (newtonsteps > AllowNewts) && (minqual < 0.7)) || (ForcedRemesh >0)          % Any large movement?
            
            [p, In] = sortrows(p,'ascend');
            types = types(In,:);
            
            [~,IA,~] = unique(p,'rows','first');
            IA = sort(IA,'ascend'); % Keep the rows stable
            p = p(IA,:);
            
            types = types(IA);
                                                    % Save current positions of p
            t = delaunay(p);                                  % List of triangles
            
            pmid = (p(t(:,1),:)+p(t(:,2),:)+p(t(:,3),:))/3;    % Compute triangle centroids
            t=t((fFunc.fd(pmid) < -geps.*fH(pmid)),:);         % Keep interior triangles only
            
            quals = simpqual(p,t);
            lowquals = find(quals <= 0.6);
            
            if (size(lowquals,1) > 0) && (ForcedRemesh == 0)
                newps = zeros(0,2);
                numlowtris  = size(lowquals,1);
                newps = (p(t(lowquals,1),:)+p(t(lowquals,2),:)+p(t(lowquals,3),:))/3; % add mid points
                p = [p; newps];
                types = [types; ones(numlowtris,1)*typeENUM.free];
                t = delaunay(p);
                
                pmid = (p(t(:,1),:)+p(t(:,2),:)+p(t(:,3),:))/3;    % Compute triangle centroids
                t=t((fFunc.fd(pmid) < -geps.*fH(pmid)),:);         % Keep interior triangles only
            end
            pold = p;
            % 4. Describe each bar by a unique pair of nodes
            bars=[t(:,[1,2]);
                  t(:,[1,3]);
                  t(:,[2,3])];         % Interior bars duplicated
            bars = sort(bars,2);
            
            %  Add the bars on the edge of the mesh to the MPC list to force a
            %  step to the edge of the solver
            
            [bars, ~, ic] = unique(bars,'rows');
            EdgeBar = find(histc(ic,min(ic):max(ic)) == 1); % find unique bars (edge only bars)
            EdgePs = unique(reshape(bars(EdgeBar,:),{},1));
            
            % Find all points on the boudary of the mesh and set them to forced snap
            
            MPCedgePs = setdiff(EdgePs,[find(types==typeENUM.mpc);find(types==typeENUM.fixed)]);
            types(MPCedgePs) = typeENUM.forcesnap;
            
            %Find all edge triangles and collect there ps including inner ps
            edgeTs = zeros(0,1);
            for EdgeP = 1:size(EdgePs,1)
                p2find = EdgePs(EdgeP);
                edgeTs = [edgeTs; find(t(:,1) == p2find);find(t(:,2) ==p2find);find(t(:,3) ==p2find)];
            end
            
            edgeTs = unique(edgeTs);
            EdgeCheckPs = unique(reshape(t(edgeTs,:),{},1),'rows'); % Create a list of all the Ps that are in edge triangles this speeds up the search process
            EdgeCheckPs = setdiff(EdgeCheckPs,find(types == typeENUM.fixed));

            % 5. Graphical output of the current mesh
            if ploton == true
                MESHplotter(p,t,MPCType)
            end
            
            if ForcedRemesh > 0
                disp(['Forced ' num2str(ForcedRemesh)])
            else
                disp('remeshed')
            end
            
            remesh = true;
            ForcedRemesh = 0;% Cb2
        end
        
        %% Snap points to boundaries if required (creates MPCs reference)
        
        % Check only Ps in edge triangles
        fDs = fFunc.fd(p(EdgeCheckPs,:));
        snapPoints = EdgeCheckPs(find(fDs >-0.5*feval(fH,p(EdgeCheckPs,:))),1);
        snapPoints = unique([snapPoints; find(types==typeENUM.forcesnap)]);
        
        %Find mpc points that are no longer close to the boudary
        NotMpcs = setdiff(find(types==typeENUM.mpc),snapPoints);
        types(NotMpcs,1) = typeENUM.free;
        
        snapPoints = setdiff(snapPoints,find(types==typeENUM.fixed));
        %%   snapPoints = setdiff(snapPoints,find(types==type.mpc));  %Removes mpc points from snapping
        if size(snapPoints,1) > 0 
            if size(find(types(snapPoints) == typeENUM.mpc),1) < size(snapPoints,1)
                types(snapPoints) = typeENUM.mpc;
                
                pSnap = p(snapPoints,:);
                dvecsnap = fFunc.distance(pSnap); % Shortest distance vec
                
                p(snapPoints,:)= pSnap+ dvecsnap;
                plotvecstobound =0;
                if plotvecstobound == 1
                    for ii = 1:size(snapPoints,1)
                        hold on
                        lclp = [pSnap(ii,:); (pSnap(ii,:)+ dvecsnap(ii,:))];
                        plot(lclp(:,1),lclp(:,2),'r-')
                    end
                end
                pold = inf;
                newtonsteps = inf;
                ForcedRemesh = 1;
                continue
            end
        end
        
        %% cb2 remove external points not part of a bar
        % Still to evaluate neccesity
        
        p2remove = setdiff(1:size(p,1),bars)';
        if size(p2remove,1) > 0
            p(p2remove,:) = [];
            types(p2remove,:) = [];
            removedpcount =  removedpcount + size(p2remove,1);
            pold = inf;
            newtonsteps = inf;
            ForcedRemesh = 2;
            continue
        end
        
    end
    %% 6. Create bar lengths
    
    if remesh == true || (mod(newtonsteps,AllowNewts)== 0)% Recalculate bar lengths every allowed newton step, this allows for quadratic convergence
        barvec=p(bars(:,1),:) - p(bars(:,2),:);              % List of bar vectors
        L=sqrt(sum(barvec.^2,2));                          % L = Bar lengths
        hbars=feval(fH,(p(bars(:,1),:)+p(bars(:,2),:))/2,varargin{:}); % Actual desired lengths
        L0=hbars*sqrt(sum(L.^2)/sum(hbars.^2)); L0act = L0;    % L0 = Desired lengths scaled by sum of the squares
        L0 = hbars; L0act = hbars; %this fails ???
        disp('LENGTH CHANGED')
    end
    
    %% CB2   Density control - remove/average points that are too close together
    
    if ForceNewts == false 
        %% short edge density control
        if ((mod(counter,densityctrlfreq)==0)  && any(L0act>destL*L)) || counter == 0 || any(L0act>3*destL*L)
            index = find(L0act>destL*L);
            ShrtLs = L(index,:);
            [~, InDx] = sort(ShrtLs,'ascend');
            shrtbars = bars(index,:);
            shrtbars = shrtbars(InDx,:);
            
            removedps = zeros(0,1);
            barnum = 0;
            
            while barnum < size(shrtbars,1)
                barnum =  barnum + 1;
                p1 = shrtbars(barnum,1);
                p2 = shrtbars(barnum,2);
                % Check if bar is to fixed node
                removedp = 0;
                if (types(p1,1) == typeENUM.fixed) && (types(p2,1) == typeENUM.fixed) % fixed fixed
                    % do nothing
                    continue
                elseif (types(p1,1) == typeENUM.fixed) && (types(p2,1) ~= typeENUM.fixed) % fixed & not fixed
                    % remove not fixed node
                    removedp = p2;
                elseif (types(p1,1) ~= typeENUM.fixed) && (types(p2,1) == typeENUM.fixed) % not fixed & fixed
                    % remove not fixed node
                    removedp = p1;
                elseif (types(p1,1) == typeENUM.mpc) && (types(p2,1) == typeENUM.mpc) % mpc &mpc
                    %take average point and make p1 as mpc and p2 removed
                    p(p1,:) = (p(p1,:)+p(p2,:))/2;
                    types(p1) = typeENUM.mpc;
                    removedp = p2;
                elseif (types(p1,1) == typeENUM.mpc) && (types(p2,1) ~= typeENUM.mpc) % mpc & not mpc
                    % leave mpc remove other point
                    removedp = p2;
                elseif (types(p1,1) ~= typeENUM.mpc) && (types(p2,1) == typeENUM.mpc) % not mpc & mpc
                    % leave mpc remove other point
                    removedp = p1;
                elseif (types(p1,1) == typeENUM.free) && (types(p2,1) == typeENUM.free) % free & free
                    %take average point and make p1 as free, p2 is removed
                    p(p1,:) = (p(p1,:)+p(p2,:))/2;
                    types(p1) = typeENUM.free;
                    removedp = p2;
                end
                
                
                if (removedp ~= 0) %overload control if zero, must not be considered
                    removedps =[removedps;removedp];
                    rowsCR = unique([find(shrtbars(barnum+1:end,1)==removedp);find(shrtbars(barnum+1:end,2)==removedp)])+ barnum;
                    
                    if size(rowsCR,1) > 0
                        shrtbars(rowsCR,:) = []; % remove rows containing the removed point
                    end
                end
                
                removedpcount = removedpcount +1;
                %rows containing the removed point     
            end
            if size(removedps,1)>0
                p(removedps,:) = [];
                types(removedps,:) = [];
                pold=inf;
                newtonsteps = inf;
                ForcedRemesh = 3;
                continue;
            end
            
        end
        
        %% Long bars edge control (add points)
        testdest = sum(L)/sum(L0act);
        testdest = mean(L./L0act);
        
        if testdest < 1.2 && (testdest > 0.7 || any(L0act*destL*2 < L))
            if ((mod(counter,densityctrlfreq)==0)  && any(L0act*destL < L)) || counter == 0 || any(L0act*destL*3 < L)
                
                index = find(L>destL*L0act);
                
                LongLs = L(index,:);
                LongL0acts = L0act(index,:);
                LongBars = bars(index,:);
                
                [LongLs,InDx] = sort(LongLs,'descend');
                LongL0acts = LongL0acts(InDx,:);
                LongBars = LongBars(InDx,:);
                
                newps = zeros(0,1);
                newTypes = zeros(0,1);
                barnum = 0;
                
                while barnum < size(LongBars,1)
                    barnum = barnum + 1;
                    p1 = LongBars(barnum,1);
                    p2 = LongBars(barnum,2);
                    %                if barnum == size(LongBars,1)
                    %                    disp('10')
                    %                end
                    NoPsToAdd = floor(LongLs(barnum,:)/(destL*LongL0acts(barnum,1)));
                    %          NoPsToAdd = floor(LongLs(barnum,:)/(LongL0acts(barnum,1)));
                    AddedPs = p(p1,:) + 1/(NoPsToAdd+1)*[1:NoPsToAdd]'.*(p(p2,:)-p(p1,:));
                    
                    %remove long bars that have an edge attached to this
                    AddedTypes = ones(NoPsToAdd,1)*typeENUM.free;
                    rowsCR = unique([find(LongBars(barnum+1:end,1)==LongBars(barnum,1)); find(LongBars(barnum+1:end,2)==LongBars(barnum,1));
                        find(LongBars(barnum+1:end,1)==LongBars(barnum,2)); find(LongBars(barnum+1:end,2)==LongBars(barnum,2))]);%+ barnum;
                    
                    if size(rowsCR,1) > 0
                        LongBars(rowsCR,:) = []; % remove rows containing the removed point
                        LongL0acts(rowsCR,:) = [];
                        LongLs(rowsCR,:) = [];
                    end
                    if size(AddedPs,1) ~= size(AddedTypes,1)
                        disp('failed to add types')
                        pause
                    end
                    newps = [newps; AddedPs];
                    newTypes = [newTypes; AddedTypes];
                end
                
                p = [p;newps];
                types = [types;newTypes];
                
                addedpcount = addedpcount+ size(newps,1);
                
                pold=inf;
                newtonsteps = inf;
                ForcedRemesh = 4;
                continue;
            end
            
        elseif testdest >= 1.2
            if ((mod(counter,densityctrlfreq)==0)  && any(L0act*destL < L)) || any(L0act*destL*3 < L)
                
                side1=sqrt(sum((p(t(:,2),:)-p(t(:,1),:)).^2,2));
                side2=sqrt(sum((p(t(:,3),:)-p(t(:,1),:)).^2,2));
                side3=sqrt(sum((p(t(:,3),:)-p(t(:,2),:)).^2,2));
                tcircs = side1+side2+side3;
                pmid = (p(t(:,1),:)+p(t(:,2),:)+p(t(:,3),:))/3;    % Compute triangle centroids
                Largets = find(tcircs./3 > destL.*fH(pmid));
                
                %      leng1 = fH(side1);
                %      leng2 = fH(side2);
                %      leng3 = fH(side3);
                %      EdgeLenArr = [leng1 leng2 leng3];
                %      Largets = find(sum(EdgeLenArr > destL.*fH(pmid),2)>2);
                
                newps = (p(t(Largets,1),:)+p(t(Largets,2),:)+p(t(Largets,3),:))/3;
                
                if size(newps,1)>0
                    p = [p; newps];
                    types = [types; ones(size(newps,1),1)*typeENUM.free];
                    
                    pold=inf;
                    newtonsteps = inf;
                    ForcedRemesh = 4;
                    continue;
                end
                
            end
        end
    end
    
    %% CB2 Start solve code variable initialisation
    if remesh == true
        % number of mpc nodes lagrange
        MPCnums = find(types==typeENUM.mpc);
        nmpc = size(MPCnums,1);
        MPCdof = sort([MPCnums*2-1;MPCnums*2]);
        
        % find number of fixed nodes
        pnums = find(types==typeENUM.fixed);
        pdof = sort([pnums*2-1;pnums*2]);
        
        % find number of free nodes
        fnums = find(types==typeENUM.free);
        fdof = sort([fnums*2-1;fnums*2]);
        ndof = size(MPCdof,1)+size(pdof,1)+size(fdof,1); %truss contribution total dofs
        
        u = zeros(ndof,1);
        
        lambda = 0;
        
        remesh = false;
        newtonsteps = 0;
    end
    
    
    
    %% Truss Consistant tangent contributions
    F = zeros(ndof,1);
    DFDU = zeros(ndof,ndof);
    
    for i = 1:size(bars,1)
        dof = [bars(i,1)*2-1, bars(i,1)*2, bars(i,2)*2-1, bars(i,2)*2]';
        L0bar = L0(i,1); %desired length of bar
        uloc = u(dof);
        x = [pold(bars(i,1),:), pold(bars(i,2),:)]';
        [dFdu,broken,Ft] = CB_2dtruss_CT(uloc,x,L0bar); % CB_2dtruss_CT has 3x methods for Bar contribution.
        
        if broken == true
            break
        end
        
        DFDU(dof,dof) = DFDU(dof,dof) + dFdu;
        
        F(dof) = F(dof) + Ft;
    end
    
    
    %% MPC methods contributions
    % These mehtods can be selected at the input parameters at the top of this script
    % La grange and Ellimination methids have seperate newton step updates
    
    if 1==MPC_lagrange %% Lagrange for circle
        mpccount = 0;
        DRmpcdUc = sparse(nmpc,ndof);
        DDRmpcddUc = zeros(ndof,ndof);
        RMpc = sparse(nmpc,1);
        
        
        if newtonsteps == 0 %just remeshed
            lambda = zeros(nmpc,1); %only used for la grange
            ldof = ((ndof+1):(ndof+nmpc))';
        end
        
        for i = MPCnums'
            mpccount = mpccount +1;
            
            dof = i*2-1:i*2;
            
            pcur = pold(i,:);
            p1 = pcur(1,1);
            p2 = pcur(1,2);
            uloc = u(dof);
            uloc1 = uloc(1,1);
            uloc2 = uloc(2,1);
            
            Point2B = [(p1 + uloc1) (p2 + uloc2)];
            
            RMpc(mpccount,1) = fFunc.F(Point2B);
            
            DRmpcdUc(mpccount,dof) = fFunc.dF(Point2B);            
            ddRddu = fFunc.ddF(Point2B);            
            DDRmpcddUc(dof,dof) = DDRmpcddUc(dof,dof)+lambda(mpccount,1)*ddRddu;
        end
        
        K_global = [(DFDU+DDRmpcddUc) DRmpcdUc'
            DRmpcdUc sparse(nmpc,nmpc)];
        
        RHS = sparse(ndof+nmpc,1);
        RHS(pdof,1) = F(pdof,1);
        RHS(fdof,1) = F(fdof,1);
        RHS(ldof,1) = RMpc;
        
        RHS(MPCdof,1) = F(MPCdof,1)+(DRmpcdUc(:,MPCdof))'*lambda;
        
        %solve dof
        solvedof = [fdof;MPCdof;ldof];
        
        % Newton update
        Supdate = zeros(ndof+nmpc,1);
        Supdate(solvedof) = -K_global(solvedof,solvedof)\RHS(solvedof);
        
        % Reshape to same as p
        update = Supdate(1:ndof,1);
        delta_lam = Supdate(ldof);
        
        %     %Min Norm update attempt
        P_update = [update(1:2:end,1), update(2:2:end,1)];
        
        % Determine proportion of update to be used
        % if any bar wants to change by more than half its length, then reduce
        % update size
        
        barvecupdate = P_update(bars(:,1),:)-P_update (bars(:,2),:);
        barchange = sqrt(sum(barvecupdate.^2,2));
        change = abs(barchange./L);
        maxchange = max(change);
        
        if maxchange > UpdateLimiter
            delta = UpdateLimiter/maxchange;
        else
            delta = 1;
        end
        
        lambda = lambda + delta*delta_lam;
        u = u + delta*update;
        p = pold+[u(1:2:end),u(2:2:end)];
        
    else
        %% Elimination methods
        %  Selection is created is done by the slave selection as the partial
        %  derivitive with the greatest constribution.
        
        Slavedofs_temp = zeros(ndof,1);
        Masterdofs_temp = zeros(ndof,1);
        Pmat_temp = zeros(ndof,ndof);
        dPdm_mat_temp = zeros(ndof,ndof);
        mpccount = 0;
        for i = MPCnums'
            mpccount = mpccount +1;
            
            dof = i*2-1:i*2;
            
            pcur = pold(i,:);
            uloc = u(dof);
            
            Point2B = pcur + uloc';
            
            dRdU = fFunc.dF(Point2B);
            
            dRdu1 = dRdU(1,1);
            dRdu2 = dRdU(1,2);
            
            if 0 == EllimACt % CB2 second partial dirivitive approximation
                ddRddu = fFunc.ddF(Point2B);
                
                ddRdu1du1 = ddRddu(1,1);
                ddRdu1du2 = ddRddu(1,2);
                ddRdu2du1 = ddRddu(2,1);
                ddRdu2du2 = ddRddu(2,2);
                
                if abs(dRdu1) >= abs(dRdu2)
                    MasterDof = dof(1,2); %master is dof2
                    SlaveDof = dof(1,1);
                    Pmatlocal = -dRdu2/dRdu1;
                    %dPmatlocal = -(ddRdu2du2*dRdu1 - dRdu2*ddRdu1du2)/(dRdu1^2); %Incorrect
                    dPmatlocal = -((ddRdu2du1*Pmatlocal+ddRdu2du2)*dRdu1 - dRdu2*(ddRdu1du1*Pmatlocal+ddRdu1du2))/(dRdu1^2);
                    
                elseif abs(dRdu1) < abs(dRdu2)
                    MasterDof = dof(1,1); %master is dof1
                    SlaveDof = dof(1,2);
                    Pmatlocal = -dRdu1/dRdu2;
                    %dPmatlocal = -(ddRdu1du1*dRdu2 - dRdu1*ddRdu2du1)/dRdu2^2; %incorrect
                    dPmatlocal = -((ddRdu1du2*Pmatlocal +  ddRdu1du1)*dRdu2 - dRdu1*(ddRdu2du2*Pmatlocal+ddRdu2du1))/dRdu2^2; %Campbell
                end
                
            else %actual second derivitive
                
                if abs(dRdu1) >= abs(dRdu2)
                    MasterDof = dof(1,2); %master is dof2
                    SlaveDof = dof(1,1);
                    %              Pmatlocal = -dRdu2/dRdu1;
                    Pmatlocal = fFunc.dFelimActual(Point2B,1);
                    dPmatlocal = fFunc.ddFelimActual(Point2B,1);
                elseif abs(dRdu1) < abs(dRdu2)
                    MasterDof = dof(1,1); %master is dof1
                    SlaveDof = dof(1,2);
                    %              Pmatlocal = -dRdu1/dRdu2;
                    Pmatlocal = fFunc.dFelimActual(Point2B,2);
                    dPmatlocal = fFunc.ddFelimActual(Point2B,2);
                end
            end
            
            Slavedofs_temp(SlaveDof,1) = 1;
            Masterdofs_temp(MasterDof,1) = 1;
            Pmat_temp(SlaveDof,MasterDof) = Pmatlocal;
            dPdm_mat_temp(MasterDof,MasterDof) = dPmatlocal*F(SlaveDof,1);
        end
        
        SlaveDofs = find(Slavedofs_temp==1);
        MasterDofs = find(Masterdofs_temp==1);
        
        %Solve degrees of freedom
        solvedof = [fdof;
            MasterDofs];
        
        Pmat = Pmat_temp(SlaveDofs,MasterDofs);
        dPdm_mat_term2 = dPdm_mat_temp(MasterDofs,MasterDofs);
        
        RHS_f = F(fdof,1);
        RHS_ms = F(MasterDofs,1) + Pmat'*F(SlaveDofs,1);
        RHS = [RHS_f;
               RHS_ms];
        
        Kff = DFDU(fdof,fdof);
        Kfm = DFDU(fdof,MasterDofs) + DFDU(fdof,SlaveDofs)*Pmat;
        Kmf = DFDU(MasterDofs,fdof) + Pmat*DFDU(SlaveDofs,fdof);
        Kmm = DFDU(MasterDofs,MasterDofs) + dPdm_mat_term2 +  Pmat'*DFDU(SlaveDofs,MasterDofs) + DFDU(MasterDofs,SlaveDofs)*Pmat + Pmat'*DFDU(SlaveDofs,SlaveDofs)*Pmat;
        
        %Elmination Method mpcs K_global
        K_global = [Kff Kfm;
            Kmf Kmm];
        
        % Newton update
        Supdate = zeros(ndof,1);
        Supdate(solvedof,1) = -K_global\RHS;%% Lagrange for circle
        
                
        for i = MPCnums'
            mpccount = mpccount +1;
            
            dof = i*2-1:i*2;
            pcurold = pold(i,:);
            ulocold = u(dof);
            
            PointPrev = pcurold + ulocold';
            
            if sum(dof(2) == MasterDofs) == 0
                lclmasterdof = 1;
                lclslavedof = 2;
            else
                lclmasterdof = 2;
                lclslavedof = 1;
            end
                
            SupdateMaster = Supdate(dof(1,lclmasterdof),1);
            
            SlaveUpdate = fFunc.FuncFelimActual(PointPrev,SupdateMaster,lclmasterdof);
            
            Supdate(dof(1,lclslavedof),1) = SlaveUpdate;
            
        end
                
%         Supdate(SlaveDofs,1) = Pmat*Supdate(MasterDofs,1);
        
        % Reshape to same as p
        update = Supdate(1:ndof,1);%% Lagrange for circle
        P_update = [update(1:2:end,1), update(2:2:end,1)];
        
        % Determine proportion of update to be used
        % if any bar wants to change by more than half its length, then reduce
        % update size
        
        barvecupdate = P_update(bars(:,1),:)-P_update (bars(:,2),:);
        barchange = sqrt(sum(barvecupdate.^2,2));
        change = abs(barchange./L);
        maxchange = max(change);
        
        if maxchange > UpdateLimiter
            delta = UpdateLimiter/maxchange;
        else
            delta = 1;
        end
        
        u = u + delta*update;
        p = pold+[u(1:2:end),u(2:2:end)];
        
    end
    
    newtonsteps = newtonsteps +1;
    
    %% end CB2
    
    % 8. Termination criterion:
    if ploton == true
        %PlotMPCs points around edge
        hold on
        plot(p(MPCnums,1),p(MPCnums,2),'r.')
        plot(p(pnums,1),p(pnums,2),'k*')
        drawnow
    end
    %     axis([-3 3 -3 3])
    axis equal
    hold off
    
    %    if max(abs(update)) < endtol
    qual = simpqual(p,t,1);
    minqual = min(qual);
    NOrmupdt = norm(update)
    
    %     if NOrmupdt < 1
    %  AllowNewts = 5;
    %  % densityctrlfreq = 10;
    %     end
    
    
    if minqual > 0.7
        ForceNewts = true;
        AllowNewts = 3;
        densityctrlfreq = 3;
    else
        AllowNewts = 3;
        densityctrlfreq = 5;
    end
    
    counter2 = counter2 +1;
    if (NOrmupdt < endtol)
        break;
    end
    
    %     if max(sqrt(sum(deltat*Ftot(d<-geps,:).^2,2))/h0)<dptol, break; end
end

sensitivityMat = zeros(size(p,1)*size(p,2),size(fFunc.Params,1));

for ParNum = 1:size(fFunc.Params,1)
    pold = p;
    depsVec = zeros(size(fFunc.Params,1),1);
    depsVec(ParNum,1) = 0.1;
    
    lambda = zeros(nmpc,1); %only used for la grange
    ldof = ((ndof+1):(ndof+nmpc))';
            
    tmpFunc = fFunc;
    tmpFunc.Params = fFunc.Params + depsVec;
    
    % number of mpc nodes lagrange
    MPCnums = find(types==typeENUM.mpc);
    nmpc = size(MPCnums,1);
    MPCdof = sort([MPCnums*2-1;MPCnums*2]);
    
% %     pold(MPCnums,:) = pold(MPCnums,:)*1.01;
    % find number of fixed nodes && freenodes
    pnums = find(types==typeENUM.fixed);
    pdof = sort([pnums*2-1;pnums*2]);
    
    % find number of free nodes
    fnums = find(types==typeENUM.free);
    fdof = sort([fnums*2-1;fnums*2]);
    
    pfnums = [pnums; fnums];
    pfdof = [pdof;fdof];
    fdof = [];
    
    ndof = size(MPCdof,1)+size(pfdof,1); %truss contribution total dofs
        
    u = zeros(ndof,1);
        
    
        
    remesh = false;
    newtonsteps = 0;
    MESHplotter(p,t,MPCType)   
    while true
        
        %% Truss Consistant tangent contributions
        F = zeros(ndof,1);
        DFDU = zeros(ndof,ndof);
        
        for i = 1:size(bars,1)
            dof = [bars(i,1)*2-1, bars(i,1)*2, bars(i,2)*2-1, bars(i,2)*2]';
            L0bar = L0(i,1); %desired length of bar
            uloc = u(dof);
            x = [p(bars(i,1),:), p(bars(i,2),:)]';
            [dFdu,broken,Ft] = CB_2dtruss_CT(uloc,x,L0bar); % CB_2dtruss_CT has 3x methods for Bar contribution.
            
            if broken == true
                break
            end
            
            DFDU(dof,dof) = DFDU(dof,dof) + dFdu;
            
            F(dof) = F(dof) + Ft;
        end
                
        %% MPC methods contributions
        % These mehtods can be selected at the input parameters at the top of this script
        % La grange and Ellimination methids have seperate newton step updates
        
        F = zeros(ndof,1);
    DFDU = zeros(ndof,ndof);
    
    for i = 1:size(bars,1)
        dof = [bars(i,1)*2-1, bars(i,1)*2, bars(i,2)*2-1, bars(i,2)*2]';
        L0bar = L0(i,1); %desired length of bar
        uloc = u(dof);
        x = [pold(bars(i,1),:), pold(bars(i,2),:)]';
        [dFdu,broken,Ft] = CB_2dtruss_CT(uloc,x,L0bar); % CB_2dtruss_CT has 3x methods for Bar contribution.
        
        if broken == true
            break
        end
        
        DFDU(dof,dof) = DFDU(dof,dof) + dFdu;
        
        F(dof) = F(dof) + Ft;
    end
    
    
    %% MPC methods contributions
    % These mehtods can be selected at the input parameters at the top of this script
    % La grange and Ellimination methids have seperate newton step updates
    
    if 1==MPC_lagrange %% Lagrange for circle
        mpccount = 0;
        DRmpcdUc = sparse(nmpc,ndof);
        DDRmpcddUc = sparse(ndof,ndof);
        RMpc = sparse(nmpc,1);

        
        for i = MPCnums'
            mpccount = mpccount +1;
            
            dof = i*2-1:i*2;
            
            pcur = pold(i,:);
            p1 = pcur(1,1);
            p2 = pcur(1,2);
            uloc = u(dof);
            uloc1 = uloc(1,1);
            uloc2 = uloc(2,1);
            
            Point2B = [(p1 + uloc1) (p2 + uloc2)];
            RMpc(mpccount,1) = tmpFunc.F(Point2B);
            
            DRmpcdUc(mpccount,dof) = tmpFunc.dF(Point2B);            
            ddRddu = tmpFunc.ddF(Point2B);            
            DDRmpcddUc(dof,dof) = DDRmpcddUc(dof,dof)+lambda(mpccount,1)*ddRddu;
        end
        
        K_global = [(DFDU+DDRmpcddUc) DRmpcdUc'
            DRmpcdUc sparse(nmpc,nmpc)];
        
        RHS = sparse(ndof+nmpc,1);
        RHS(pdof,1) = F(pdof,1);
        RHS(fdof,1) = F(fdof,1);
        RHS(ldof,1) = RMpc;
        
        RHS(MPCdof,1) = F(MPCdof,1)+(DRmpcdUc(:,MPCdof))'*lambda;
        
        %solve dof
        solvedof = [fdof;MPCdof;ldof];
        
        % Newton update
        Supdate = zeros(ndof+nmpc,1);
        Supdate(solvedof) = -K_global(solvedof,solvedof)\RHS(solvedof);
        
        update = Supdate(1:ndof,1);
        delta_lam = Supdate(ldof,1);
        
        %     %Min Norm update attempt
        P_update = [update(1:2:end,1), update(2:2:end,1)];
        
        % Determine proportion of update to be used
        % if any bar wants to change by more than half its length, then reduce
        % update size
        
        barvecupdate = P_update(bars(:,1),:)-P_update (bars(:,2),:);
        barchange = sqrt(sum(barvecupdate.^2,2));
        change = abs(barchange./L);
        maxchange = max(change);
        
        if maxchange > UpdateLimiter
            delta = UpdateLimiter/maxchange;
        else
            delta = 1;
        end
        
        lambda = lambda + delta*delta_lam;
        u = u + delta*update;
    else
        %% Elimination methods
        %  Selection is created is done by the slave selection as the partial
        %  derivitive with the greatest constribution.
        
        Slavedofs_temp = zeros(ndof,1);
        Masterdofs_temp = zeros(ndof,1);
        Pmat_temp = zeros(ndof,ndof);
        dPdm_mat_temp = zeros(ndof,ndof);
        mpccount = 0;
        for i = MPCnums'
            mpccount = mpccount +1;
            
            dof = i*2-1:i*2;
            
            pcur = pold(i,:);
            uloc = u(dof);
            
            Point2B = pcur + uloc';
            
            dRdU = tmpFunc.dF(Point2B);
            
            dRdu1 = dRdU(1,1);
            dRdu2 = dRdU(1,2);
            
            if 0 == EllimACt % CB2 second partial dirivitive approximation
                 ddRddu = fFunc.ddF(Point2B);
                
                ddRdu1du1 = ddRddu(1,1);
                ddRdu1du2 = ddRddu(1,2);
                ddRdu2du1 = ddRddu(2,1);
                ddRdu2du2 = ddRddu(2,2);
                
                if abs(dRdu1) >= abs(dRdu2)
                    MasterDof = dof(1,2); %master is dof2
                    SlaveDof = dof(1,1);
                    Pmatlocal = -dRdu2/dRdu1;
                    %dPmatlocal = -(ddRdu2du2*dRdu1 - dRdu2*ddRdu1du2)/(dRdu1^2); %Incorrect
                    dPmatlocal = -((ddRdu2du1*Pmatlocal+ddRdu2du2)*dRdu1 - dRdu2*(ddRdu1du1*Pmatlocal+ddRdu1du2))/(dRdu1^2);
                    
                elseif abs(dRdu1) < abs(dRdu2)
                    MasterDof = dof(1,1); %master is dof1
                    SlaveDof = dof(1,2);
                    Pmatlocal = -dRdu1/dRdu2;
                    %dPmatlocal = -(ddRdu1du1*dRdu2 - dRdu1*ddRdu2du1)/dRdu2^2; %incorrect
                    dPmatlocal = -((ddRdu1du2*Pmatlocal +  ddRdu1du1)*dRdu2 - dRdu1*(ddRdu2du2*Pmatlocal+ddRdu2du1))/dRdu2^2; %Campbell
                end
                
            else %actual second derivitive
                
                if abs(dRdu1) >= abs(dRdu2)
                    MasterDof = dof(1,2); %master is dof2
                    SlaveDof = dof(1,1);
                    %              Pmatlocal = -dRdu2/dRdu1;
                    Pmatlocal = tmpFunc.dFelimActual(Point2B,1);
                    dPmatlocal = tmpFunc.ddFelimActual(Point2B,1);
                elseif abs(dRdu1) < abs(dRdu2)
                    MasterDof = dof(1,1); %master is dof1
                    SlaveDof = dof(1,2);
                    %              Pmatlocal = -dRdu1/dRdu2;
                    Pmatlocal = tmpFunc.dFelimActual(Point2B,2);
                    dPmatlocal = tmpFunc.ddFelimActual(Point2B,2);
                end
            end
            
            Slavedofs_temp(SlaveDof,1) = 1;
            Masterdofs_temp(MasterDof,1) = 1;
            Pmat_temp(SlaveDof,MasterDof) = Pmatlocal;
            dPdm_mat_temp(MasterDof,MasterDof) = dPmatlocal*F(SlaveDof,1);
        end
        
        SlaveDofs = find(Slavedofs_temp==1);
        MasterDofs = find(Masterdofs_temp==1);
        
        %Solve degrees of freedom
        solvedof = [fdof;
            MasterDofs];
        
        Pmat = Pmat_temp(SlaveDofs,MasterDofs);
        dPdm_mat_term2 = dPdm_mat_temp(MasterDofs,MasterDofs);
        
        RHS_f = F(fdof,1);
        RHS_ms = F(MasterDofs,1) + Pmat'*F(SlaveDofs,1);
        RHS = [RHS_f;
            RHS_ms];
        
        Kff = DFDU(fdof,fdof);
        Kfm = DFDU(fdof,MasterDofs) + DFDU(fdof,SlaveDofs)*Pmat;
        Kmf = DFDU(MasterDofs,fdof) + Pmat*DFDU(SlaveDofs,fdof);
        Kmm = DFDU(MasterDofs,MasterDofs) + dPdm_mat_term2 +  Pmat'*DFDU(SlaveDofs,MasterDofs) + DFDU(MasterDofs,SlaveDofs)*Pmat + Pmat'*DFDU(SlaveDofs,SlaveDofs)*Pmat;
        
        %Elmination Method mpcs K_global
        K_global = [Kff Kfm;
            Kmf Kmm];
        
        % Newton update
        Supdate = zeros(ndof,1);
        Supdate(solvedof,1) = -K_global\RHS;%% Lagrange for circle
        
        
        for i = MPCnums'
            mpccount = mpccount +1;
            
            dof = i*2-1:i*2;
            pcurold = pold(i,:);
            ulocold = u(dof);
            
            PointPrev = pcurold + ulocold';
            
            if sum(dof(2) == MasterDofs) == 0
                lclmasterdof = 1;
                lclslavedof = 2;
            else
                lclmasterdof = 2;
                lclslavedof = 1;
            end
            
            SupdateMaster = Supdate(dof(1,lclmasterdof),1);
            
            SlaveUpdate = tmpFunc.FuncFelimActual(PointPrev,SupdateMaster,lclmasterdof);
            
            Supdate(dof(1,lclslavedof),1) = SlaveUpdate;
            
        end
        
        %         Supdate(SlaveDofs,1) = Pmat*Supdate(MasterDofs,1);
        update = Supdate(1:ndof,1);
        
        
        %     %Min Norm update attempt
        P_update = [update(1:2:end,1), update(2:2:end,1)];
        
        % Determine proportion of update to be used
        % if any bar wants to change by more than half its length, then reduce
        % update size
        
        barvecupdate = P_update(bars(:,1),:)-P_update (bars(:,2),:);
        barchange = sqrt(sum(barvecupdate.^2,2));
        change = abs(barchange./L);
        maxchange = max(change);
        
        if maxchange > UpdateLimiter
            delta = UpdateLimiter/maxchange;
        else
            delta = 1;
        end
        
        u = u + delta*update;
    end
    
    ps2 = pold+[u(1:2:end),u(2:2:end)];
    
    if ploton == true
        %PlotMPCs points around edge
        hold on
        plot(ps2(MPCnums,1),ps2(MPCnums,2),'b.')
        plot(ps2(pfnums,1),ps2(pfnums,2),'g*')
        drawnow
    end
    
    NOrmupdt = norm(update)
    
    if NOrmupdt < 10e-8;
        break
    end
    end
    
    sensitivityMat(:,ParNum) = u;
    
end

%%

MESHplotter(p,t,MPCType)

minqual = min(qual)

npsend = size(p,1)
removedpcount
addedpcount
iters =counter

lr = L./L0act;

minls = min(lr)
maxls = max(lr)
avelr = mean(lr)
time = toc

% pause

% Clean up and plot final mesh
% % % [p,t]=fixmesh(p,t);
% % % simpplot(p,t)
end

function [a] = MESHplotter(p,t,fignum)
figure(fignum)
cla,patch('vertices',p,'faces',t,'edgecol','k','facecol',[.8,.9,1]);
drawnow

% draw low quality elements
qual = simpqual(p,t,1);
AD = find(qual<.8);
hold on
patch('vertices',p,'faces',t(AD,:),'edgecol','k','facecol','g'); drawnow;
AD = find(qual<0.4);
patch('vertices',p,'faces',t(AD,:),'edgecol','k','facecol','c'); drawnow;
axis equal
AD = find(qual<0.4);
patch('vertices',p,'faces',t(AD,:),'edgecol','k','facecol','c'); drawnow;
axis equal
hold off
end


