function [p,t,iters,updates,AverageLcLdes,meshedatiter]=CB_distmesh2d(fd,fh,h0,bbox,pfix,endtol,TrussType,TrussS,scale,varargin)
%DISTMESH2D 2-D Mesh Generator using Distance Functions.
%   [P,T]=DISTMESH2D(FD,FH,H0,BBOX,PFIX,FPARAMS)
%
%      P:         Node positions (Nx2)
%      T:         Triangle indices (NTx3)
%      FD:        Distance function d(x,y)
%      FH:        Scaled edge length function h(x,y)
%      H0:        Initial edge length
%      BBOX:      Bounding box [xmin,ymin; xmax,ymax]
%      PFIX:      Fixed node positions (NFIXx2)
%      FPARAMS:   Additional parameters passed to FD and FH
%
%   Example: (Uniform Mesh on Unit Circle)
%      fd=@(p) sqrt(sum(p.^2,2))-1;
%      [p,t]=distmesh2d(fd,@huniform,0.2,[-1,-1;1,1],[]);
%
%   Example: (Rectangle with circular hole, refined at circle boundary)
%      fd=@(p) ddiff(drectangle(p,-1,1,-1,1),dcircle(p,0,0,0.5));
%      fh=@(p) 0.05+0.3*dcircle(p,0,0,0.5);
%      [p,t]=distmesh2d(fd,fh,0.05,[-1,-1;1,1],[-1,-1;-1,1;1,-1;1,1]);
%
%   Example: (Polygon)
%      pv=[-0.4 -0.5;0.4 -0.2;0.4 -0.7;1.5 -0.4;0.9 0.1;
%          1.6 0.8;0.5 0.5;0.2 1;0.1 0.4;-0.7 0.7;-0.4 -0.5];
%      [p,t]=distmesh2d(@dpoly,@huniform,0.1,[-1,-1; 2,1],pv,pv);
%
%   Example: (Ellipse)
%      fd=@(p) p(:,1).^2/2^2+p(:,2).^2/1^2-1;
%      [p,t]=distmesh2d(fd,@huniform,0.2,[-2,-1;2,1],[]);
%
%   Example: (Square, with size function point and line sources)
%      fd=@(p) drectangle(p,0,1,0,1);
%      fh=@(p) min(min(0.01+0.3*abs(dcircle(p,0,0,0)), ...
%                   0.025+0.3*abs(dpoly(p,[0.3,0.7; 0.7,0.5]))),0.15);
%      [p,t]=distmesh2d(fd,fh,0.01,[0,0;1,1],[0,0;1,0;0,1;1,1]);
%
%   Example: (NACA0012 airfoil)
%      hlead=0.01; htrail=0.04; hmax=2; circx=2; circr=4;
%      a=.12/.2*[0.2969,-0.1260,-0.3516,0.2843,-0.1036];
%
%      fd=@(p) ddiff(dcircle(p,circx,0,circr),(abs(p(:,2))-polyval([a(5:-1:2),0],p(:,1))).^2-a(1)^2*p(:,1));
%      fh=@(p) min(min(hlead+0.3*dcircle(p,0,0,0),htrail+0.3*dcircle(p,1,0,0)),hmax);
%
%      fixx=1-htrail*cumsum(1.3.^(0:4)');
%      fixy=a(1)*sqrt(fixx)+polyval([a(5:-1:2),0],fixx);
%      fix=[[circx+[-1,1,0,0]*circr; 0,0,circr*[-1,1]]'; 0,0; 1,0; fixx,fixy; fixx,-fixy];
%      box=[circx-circr,-circr; circx+circr,circr];
%      h0=min([hlead,htrail,hmax]);
%
%      [p,t]=distmesh2d(fd,fh,h0,box,fix);

%
%   See also: MESHDEMO2D, DISTMESHND, DELAUNAYN, TRIMESH.

%   distmesh2d.m v1.1
%   Copyright (C) 2004-2012 Per-Olof Persson. See COPYRIGHT.TXT for details.

dptol=.001; ttol=.1; Fscale=1.2; deltat=.2; geps=-.7*h0; deps=sqrt(eps)*h0;
densityctrlfreq=4; %CB2

% 1. Create initial distribution in bounding box (equilateral triangles)
[x,y]=meshgrid(bbox(1,1):h0:bbox(2,1),bbox(1,2):h0*sqrt(3)/2:bbox(2,2));
x(2:2:end,:)=x(2:2:end,:)+h0/2;                      % Shift even rows
p=[x(:),y(:)];                                       % List of node coordinates
t=delaunayn(p);                                  % List of triangles
pmid=(p(t(:,1),:)+p(t(:,2),:)+p(t(:,3),:))/3;    % Compute centroids
% t=t(feval(fd,pmid,varargin{:})<-geps,:);         % Keep interior triangles
        
% 2. Remove points outside the region, apply the rejection method
p=p(feval(fd,p,varargin{:})<geps,:);                 % Keep only d<0 points
% r0=1./feval(fh,p,varargin{:}).^2;                    % Probability to keep point
% p=p(rand(size(p,1),1)<r0./max(r0),:); 
% Rejection method

pfixbound = pfix;;
 if ~isempty(pfix), p=setdiff(p,pfix,'rows'); end     % Remove duplicated nodes
pfix=unique(pfix,'rows'); nfix=size(pfix,1);         % Ensure unique fixed nodes
p=[pfix; p];                                         % Prepend fix points
p = unique(p,'rows','stable');

count=0;
pold=inf;                                            % For first iteration
t = inf;
normupdate = inf;
% % % clf,view(2),axis equal,axis off
updates = zeros(0,1);
while 1
    count=count+1;
    meshedatiter(count,1) = 0;
    % 3. Retriangulation by the Delaunay algorithm
    if max(sqrt(sum((p-pold).^2,2))/h0)>ttol %&& normupdate>.1 % && mod(count,3) ==1            % Any large movement?
        test = sum(sum(isnan(p)));
       
        if test > 0;
           break; 
        end
        pold=p;                                          % Save current positions
        told = t;
        t=delaunayn(p);                                  % List of triangles
        pmid=(p(t(:,1),:)+p(t(:,2),:)+p(t(:,3),:))/3;    % Compute centroids
        t=t(feval(fd,pmid,varargin{:})<-geps,:);        % Keep interior triangles
        if size(told,1)~= size(t,1)
          meshedatiter(count,1) = 1;
        else
           if sum(any((t-told) ~=0)) > 0
               meshedatiter(count,1) = 1;
           end
        end    
               
        % 4. Describe each bar by a unique pair of nodes
        bars=[t(:,[1,2]);t(:,[1,3]);t(:,[2,3])];         % Interior bars duplicated
        bars=unique(sort(bars,2),'rows');                % Bars as node pairs
        % 5. Graphical output of the current mesh
       figure(11)
        hold on,cla
        plot([pfixbound(:,1); pfixbound(1,1)],[pfixbound(:,2); pfixbound(1,2)],'r','LineWidth',2)
        plot((1+geps)*[pfixbound(:,1); pfixbound(1,1)],(1+geps)*[pfixbound(:,2); pfixbound(1,2)],'g','LineWidth',2)
        plot(pfix(:,1),pfix(:,2) ,'k*')
        patch('vertices',p,'faces',t,'edgecol','k','facecol',[.8,.9,1]);

        plot([pfixbound(:,1); pfixbound(1,1)],[pfixbound(:,2); pfixbound(1,2)],'r','LineWidth',2)
        plot((1+geps)*[pfixbound(:,1); pfixbound(1,1)],(1+geps)*[pfixbound(:,2); pfixbound(1,2)],'g','LineWidth',2)
        plot(pfix(:,1),pfix(:,2) ,'k*')
        legend('Circle boundary','Point elmination boundary','Fixed boundary nodes')
        minqual = min(simpqual(p,t));
        title({'Bounding box seed Mesh seed',['Minimum quality = ' num2str(minqual)]})
        axis square
        drawnow
          remesh = true; % Cb2
    end
      figure(11)
        cla,patch('vertices',p,'faces',t,'edgecol','k','facecol',[.8,.9,1]);
        hold on
        plot(pfix(:,1),pfix(:,2),'k*');
         title({'Truss Stiffness Formula : Convergence',['Scale = L_c/L_d_e_s = ' num2str(scale)],['Minimum quality = ' num2str(minqual)]},'FontSize',16)
      drawnow
    % 6. Move m
    % 6. Move mesh points based on bar lengths L and forces F
    barvec=p(bars(:,1),:)-p(bars(:,2),:);              % List of bar vectors
    Lc=sqrt(sum(barvec.^2,2));                          % L = Bar lengths
    hbars=feval(fh,(p(bars(:,1),:)+p(bars(:,2),:))/2,varargin{:});
    Ldes=hbars; % L0 = Desired lengths
    % Density control - remove points that are too close
%     if mod(count,densityctrlfreq)==0 & any(L0>2*L)
%         p(setdiff(reshape(bars(L0>2*L,:),[],1),1:nfix),:)=[];
%         N=size(p,1); pold=inf;
%         continue;
%     end
    
    
    %% CB2 Start code
    if remesh == true
        ndof = size(p,1)*2;
        u = zeros(ndof,1);
        
        pdof = 1:nfix*2; %Prescribed DOF
        fdof = (nfix*2+1):ndof; % Free DOF
        
        remesh = false;
    end
    F= zeros(ndof,1);
    DFDU = sparse(ndof,ndof);
    for i = 1:size(bars,1)
        dof = [bars(i,1)*2-1, bars(i,1)*2, bars(i,2)*2-1, bars(i,2)*2]';
        L0bar = Ldes(i,1); %desired length of bar
        uloc = u(dof);
        x = [pold(bars(i,1),:), pold(bars(i,2),:)]';
        [dFdu,broken,Ft] = CB_2dtruss_CT(uloc,x,L0bar,TrussType,TrussS); % CB_2dtruss_CT has 3x methods for Bar contribution.
        
        if broken == true
            break
        end
        
        DFDU(dof,dof) = DFDU(dof,dof) + dFdu;
        
        F(dof) = F(dof) + Ft;
    end
    

    % Newton update
    update = zeros(ndof,1);
    update(fdof) = -DFDU(fdof,fdof)\F(fdof);
    
    % Reshape to same as p
    P_update = [update(1:2:end,1), update(2:2:end,1)];
    
    % Determine proportion of update to be used
    % if any bar wants to change by more than half its length, then reduce
    % update size
   
    barvecupdate = P_update(bars(:,1),:)-P_update (bars(:,2),:);
    barchange = sqrt(sum(barvecupdate.^2,2));
    change = abs(barchange./Lc);
    maxchange = max(change);
    alpha = 200000;
    
    if maxchange > alpha
        delta = alpha/maxchange;
    else
        delta = 1;
    end
    
    u = u + delta*update;
    p = pold+[u(1:2:end),u(2:2:end)];
    
    NormUpdate = norm(update)
    updates(size(updates,1)+1,1) = NormUpdate;
    if NormUpdate < endtol
        break;
    end
    
    if NormUpdate > 200 || isnan(NormUpdate)
        break;
    end
    if count > 50
        break;
    end
    %     if max(sqrt(sum(deltat*Ftot(d<-geps,:).^2,2))/h0)<dptol, break; end
end
iters = count;
barvec=p(bars(:,1),:)-p(bars(:,2),:);              % List of bar vectors
    Lc=sqrt(sum(barvec.^2,2));                          % L = Bar lengths
    hbars=feval(fh,(p(bars(:,1),:)+p(bars(:,2),:))/2,varargin{:});
    Ldes=hbars; % L0 = Desired lengths
AverageLcLdes = mean(Lc./Ldes);



% Clean up and plot final mesh
% % % [p,t]=fixmesh(p,t);
% % % simpplot(p,t)
